""" infinihost v{version}
INFINIDAT Host PowerTools

Usage:
    infinihost settings check [(--fix | --auto-fix)] [--with-qr-codes]
    infinihost system list
    infinihost connectivity list [--json]
    infinihost pool list [--units=UNITS] [--rescan] [--system=SYSTEM]
    infinihost volume list [(--summary | --detailed | --json)] [--units=UNITS] [--rescan]
    infinihost rescan
    infinihost logs collect [(--interactive | --only-infinihost-logs)] [--file=FILE] [--timestamp=DATETIME] [--span=TIME]
    infinihost credentials set [--system=SYSTEM] [<username>] [<password>]
    infinihost credentials get
    infinihost credentials delete

Options:
    <size>                      desired volume size (examples: 10gb, 100mb, 1tb)
    --auto-fix                  automatically attempt to fix all issues
    --detailed                  a much more detailed view
    --duration=TIME             time (in seconds) to run [default: 60]
    --fix                       ask for confirmation on each fix that can be applied
    --interactive               interactively choose what to collect
    --json                      JSON-formatted output, useful for scripting
    --only-infinihost-logs      only collect infinihost log files
    --rescan                    rescan the storage stack before listing
    --span=TIME                 number of seconds to go back from --timestamp to start collecting the logs [default: 86400]
    --summary                   a very short summary
    --system=SYSTEM             system serial or name to use, see infinihost system list for the available options [default: autodetect]
    --timestamp=DATETIME        date and time of logs to collect. see --span [default: now]
    --units=UNITS               size unit to use in 'brief' output [default: GB]
    --with-qr-codes             display QR codes in addition to links
    -h --help                   show this screen.
    -v --version                show version.

{privileges_text}{autocompletion_text}

More information:
    infinihost settings check          validate configuration of the storage stack
    infinihost connectivity list       show fiberchannel connectivity
    infinihost pool list               show attached infinibox pools
    infinihost volume list             show attached infinibox volumes
    infinihost system list             show information on connected systems
    infinihost rescan                  rescan the storage stack
    infinihost logs collect            collect diagnostic information
    infinihost credentials set         save credentials for a specific infinibox machine or globally
    infinihost credentials get         show all saved credentials
    infinihost credentials delete      remove all saved credentials

This host has OpenStack installed, and thus needs to be provisioned by OpenStack.

Getting started:

# Check that this host is ready:
{prefix}infinihost settings check

# Make the host ready:
{prefix}infinihost settings check --auto-fix

# Provision volumes with OpenStack
"""


def is_openstack_cinder_installed():
    from os import path, environ
    openstack_executables = ('cinder', 'nova')
    return any(path.exists(path.join(path.sep, "usr", "bin", basename)) for basename in openstack_executables) or \
           bool(environ.get("MOCK_CINDER"))
