from __future__ import print_function

def get_aix_tl():
    from infi.execute import execute_assert_success
    oslevel_output = execute_assert_success(["oslevel", "-r"]).get_stdout()
    return int(oslevel_output.strip().split("-")[1])

def check_os_support():
    from infi.os_info import get_platform_string
    from infi.vendata.powertools.conf import get_configuration
    case_config = get_configuration().get('readiness.cases.os_support')
    platform_tuple = get_platform_string().split("-")
    os = platform_tuple[0]
    if os == "linux":
        from platform import linux_distribution
        dist = platform_tuple[1]
        _, ver, _ = linux_distribution()
        ver = '.'.join(ver.split('.')[:2])
        supported_versions = case_config["linux"].get(dist, None)
        if not supported_versions:
            _print_warning("Linux distribution {} is not supported.".format(dist))
            return False
        if ver not in supported_versions:
            _print_warning("{} {} is not supported.".format(dist, ver))
            return False
    elif os == "windows":
        from infi.winver import Windows
        win_ver = Windows().version
        sp = Windows().service_pack
        supported_sps = case_config["windows"].get(win_ver, None)
        if not supported_sps:
            _print_warning("{} is not supported.".format(win_ver))
            return False
        if sp not in supported_sps:
            _print_warning("{} sp {} is not supported.".format(win_ver, sp))
            return False
    elif os == "solaris":
        ver, arch = platform_tuple[1], platform_tuple[2]
        supported_archs = case_config["solaris"].get(ver, None)
        if not supported_archs:
            _print_warning("Solaris version {} is not supported.".format(ver))
            return False
        if arch not in supported_archs:
            _print_warning("Solaris {} is not supported on {} architecture.".format(ver, arch))
            return False
    elif os == "aix":
        ver, arch = platform_tuple[1], platform_tuple[2]
        tl = get_aix_tl()
        ver = "{} TL {}".format(ver, tl)
        supported_archs = case_config["aix"].get(ver, None)
        if not supported_archs:
            _print_warning("AIX version {} is not supported.".format(ver))
            return False
        if arch not in supported_archs:
            _print_warning("AIX {} is not supported on {} architecture.".format(ver, arch))
            return False
    else:
        _print_warning("OS {} is not supported.".format(os))
        return False
    return True

def _print_warning(msg):
    from infi.vendata.powertools.settings.runner import get_fore_color
    from sys import stderr
    print(get_fore_color("YELLOW") + "WARNING: " + get_fore_color("RESET") + msg, file=stderr)
