""" infinihost v{version}
INFINIDAT Host PowerTools

Usage:
    infinihost settings check [(--fix | --auto-fix)] [--with-qr-codes]
    infinihost system list
    infinihost system register [(--fix | --auto-fix)] [--hostname=HOSTNAME] [--system=SYSTEM]
    infinihost connectivity list [--json]
    infinihost pool list [--units=UNITS] [--rescan] [--system=SYSTEM]
    infinihost volume list [(--summary | --detailed | --json)] [--units=UNITS] [--rescan]
    infinihost volume provision <size> [--yes] [--name=NAME] [--thin=THIN] [--pool=POOL] [--system=SYSTEM]
    infinihost volume resize <device-or-mount> <size> [--expand-by] [--yes]
    infinihost volume unmap <device-or-mount> [--yes]
    infinihost volume map <volume> [--system=SYSTEM] [--yes]
    infinihost volume rename <device-or-mount> <name>
    infinihost volume speedtest <device-or-mount> [--duration=TIME]
    infinihost metadata sync
    infinihost rescan
    infinihost logs collect [(--interactive | --only-infinihost-logs)] [--file=FILE] [--timestamp=DATETIME] [--span=TIME]
    infinihost credentials set [--system=SYSTEM] [<username> [<password>]]
    infinihost credentials get
    infinihost credentials delete

Options:
    <device-or-mount>           device to work with - accepts device path, name, or mount point
    <device>                    device to work with - accepts device path
    <mount-path>                new path for the filesystem mount [default: autodetect]
    <size>                      desired volume size (examples: 10gb, 100mb, 1tb)
    <volume>                    volume id or name to map
    --auto-fix                  automatically attempt to fix all issues
    --detailed                  a much more detailed view
    --duration=TIME             time (in seconds) to run [default: 60]
    --expand-by                 interpret the given size as an addition to the current volume size
    --file=FILE                 output filename or path (optional)
    --fix                       ask for confirmation on each fix that can be applied
    --hostname=HOSTNAME         hostname to use when registering [default: autodetect]
    --interactive               interactively choose what to collect
    --json                      JSON-formatted output, useful for scripting
    --name=NAME                 name of the volume to create [default: autodetect]
    --only-infinihost-logs      only collect infinihost log files
    --pool=POOL                 pool to provision from, see infinihost system list for the available options [default: autodetect]
    --rescan                    rescan the storage stack before listing
    --span=TIME                 number of seconds to go back from --timestamp to start collecting the logs [default: 86400]
    --summary                   a very short summary
    --system=SYSTEM             system serial or name to use, see infinihost system list for the available options [default: autodetect]
    --thin=THIN                 volume thin provisioning (options: yes, no) [default: no]
    --timestamp=DATETIME        date and time of logs to collect. see --span [default: now]
    --units=UNITS               size unit to use in 'brief' output [default: GB]
    --with-qr-codes             display QR codes in addition to links
    --yes                       provide to confirm the action
    -h --help                   show this screen.
    -v --version                show version.

{privileges_text}{autocompletion_text}

More information:
    infinihost settings check          validate configuration of the storage stack
    infinihost connectivity list       show fiberchannel connectivity
    infinihost pool list               show attached infinibox pools
    infinihost volume list             show attached infinibox volumes; does a rescan beforehand by default
    infinihost volume provision        provision a new volume from infinibox
    infinihost volume resize           change the size of an existing volume
    infinihost volume unmap            unmap a volume from infinibox
    infinihost volume speedtest        run a non-distruptive speed test on a specific volume
    infinihost system list             show information on connected systems
    infinihost system register         register host in system
    infinihost rescan                  rescan the storage stack
    infinihost logs collect            collect diagnostic information
    infinihost metadata sync           updates metadata of host and volumes on the infinibox machine
    infinihost credentials set         save credentials for a specific infinibox machine or globally
    infinihost credentials get         show all saved credentials
    infinihost credentials delete      remove all saved credentials

Getting started:

This host has Veritas multipathing installed, and thus filesystem and mounts won't be created for new volumes.

# Check that this host is ready:
{prefix}infinihost settings check

# Make the host ready:
{prefix}infinihost settings check --auto-fix

# Register the host in the system:
{prefix}infinihost system register

# Provision a new volume, a 10GB in this example:
{prefix}infinihost volume provision 10gb --yes

# You can see it by running:
{prefix}infinihost volume list
"""


from infi.vendata.powertools.utils import is_veritas_multipathing_installed


def is_veritas_installed():
    return is_veritas_multipathing_installed()
