actions_modal = {
    opresult_modal: function(json) {
        $("#opresult-modal-body-result-text").text(json.return_value.result);
        $("#opresult-modal-body-more-info").html(json.return_value.info_str);
        $("#opresult-modal-body-qr-img").attr("src", json.return_value.kb_img_url);
        $("#opresult-modal").dialog({
            modal: true,
            width: 540,
            height: 230,
            resizable: false,
            buttons: {
                OK: function() {
                    $(this).dialog("close");
                }
            }
        });
    },

    action_click: function(event) {
        var action_selector = $(event.delegateTarget);
        var action_description = action_selector.parent().next().text();
        var result_icon_src = $("img", action_selector).attr("src");
        $("#opresult-modal-icon").attr("src", result_icon_src);
        var ajaxOptions = {type: "POST",
                       dataType: "json",
                       data: {action_description: action_description},
                       cache: false,
                       url: actions_modal.opinfo_url};
        view.spinAjax(ajaxOptions, actions_modal.opresult_modal);
    },

    show : function(modal_title, msg_on_error, is_cancellable,
                    get_action_url, perform_action_url, ajax_data, done_callback, opinfo_url) {
        // modal_title          title for the dialog
        // msg_on_error         message to display in case of failure
        // is_cancellable       whether the user can cancel the action
        // get_action_url       expected to return a list of action description strings
        // perform_action_url   called to perform the next action on the list; gets its description as action_description
        //                      expected to return 'ok' / 'fail' / 'error' / 'requires reboot' / 'warning' / 'skip'
        //                      or to return an error message to abort everything
        // ajax_data            data to send to the get_action_url and perform_action_url
        // done_callback        function to call when finished (optional)
        // opinfo_url           called to receive more information about each failed action (optional)
        actions_modal.result = "ok";
        actions_modal.current_action = undefined;
        actions_modal.is_cancellable = is_cancellable;
        actions_modal.get_action_url = get_action_url;
        actions_modal.perform_action_url = perform_action_url;
        actions_modal.ajax_data = ajax_data;
        actions_modal.done_callback = done_callback;
        actions_modal.opinfo_url = opinfo_url;
        actions_modal.msg_on_error = msg_on_error;
        $("#actions-modal-hint").hide();
        $("#actions-modal .non-cancellable-element").toggle(!is_cancellable);
        var buttons = [{
                        id: "actions-modal-start",
                        text: "START",
                        click: actions_modal.startActions
                       }];
        if (is_cancellable) {
            buttons.push({
                            id: "actions-modal-cancel",
                            text: "CANCEL",
                            click: function() {
                                $("#actions-modal").dialog("close");
                            }
                         });
        }
        actions_modal.preloadIcons();
        actions_modal.set_table_ajax(function() {
            $("#actions-modal").dialog({
                modal: true,
                title: modal_title,
                width: 640,
                height: actions_modal.calc_height(),
                resizable: false,
                dialogClass: is_cancellable ? "" : "no-close",
                closeOnEscape: is_cancellable,
                buttons: buttons
            });
            $('#actions-table-scroller').scrollTop(0);
        });
    },

    set_table_ajax : function(onsuccess) {
        var table = $("#actions-table");
        table.empty();
        actions_modal.actions = [];
        view.spinAjax({url: actions_modal.get_action_url, type: "POST", data: actions_modal.ajax_data},
                  function(json) {
                      for (var i = 0; i < json.return_value.length; i++) {
                          var tr = $('<tr></tr>');
                          table.append(tr);
                          tr.append('<td><div class="action"><img src="/static/images/icon-empty.png"></div></td>');
                          tr.append('<td>' + json.return_value[i] + '</td>');
                          actions_modal.actions.push(tr);
                      }
                      onsuccess();
                  },
                  function(json) {
                      if (json == null) {
                        return false;
                      }
                      view.jsonError(json, "ERROR: " + actions_modal.msg_on_error);
                      return true; // this tells view.ajax not to show the default error modal
                  }
        );
    },

    unspin : function(selector) {
        function callback() {
            var classSelector = "." + spinnerFactory.getDefaults().className;
            $(classSelector, selector).hide();
        };
        return callback;
    },

    messageIcons: {'ok':              "/static/images/icon-good.png",
                   'fail':            "/static/images/icon-fail.png",
                   'error':           "/static/images/icon-fail.png",
                   'requires reboot': "/static/images/icon-notgood.png",
                   'warning':         "/static/images/icon-notgood.png",
                   'skip':            "/static/images/icon-skipped.png"},

    preloadIcons: function() {
        $.each(actions_modal.messageIcons, function(key, value) {
            $('<img/>')[0].src = value;
        })
    },

    setActionResult: function(result) {
        var new_img = actions_modal.messageIcons[result];
        $("img", actions_modal.current_action).attr("src", new_img);

        if (result != "ok" && result != "skip" && result != "warning") {
            if (actions_modal.result != "fail") {
                actions_modal.result = result;
            }
        };

        if (result != 'ok' && actions_modal.opinfo_url) {
            $("#actions-modal-hint").show();
            actions_modal.current_action.click(actions_modal.action_click);
            actions_modal.current_action.css("cursor", "help");
        }
    },

    startActions : function() {
        $("#actions-modal-start").button("disable");
        $("#actions-modal-cancel").button("disable");
        $("#actions-modal").dialog("option", {
            dialogClass: "no-close",
            closeOnEscape: false
        });
        actions_modal.nextAction();
    },

    actionComplete : function(json) {
        actions_modal.setActionResult(json.return_value);
        actions_modal.nextAction();
    },

    actionsDone: function() {
        $("#actions-modal").dialog("option", "buttons", [
            {
                id: "actions-modal-done",
                text: "DONE",
                click: actions_modal.dismiss
            }
        ]);
        $('#actions-modal-done').focus();
    },

    dismiss: function() {
        if (actions_modal.result == "ok" || actions_modal.is_cancellable) {
            if (actions_modal.done_callback) actions_modal.done_callback();
            $("#actions-modal").dialog("close");
        }
        else {
            var desc;
            if (actions_modal.result == "requires reboot") {
                desc = "Please reboot the host to complete the operation.";
                showError("ERROR", "Cannot continue: " + actions_modal.msg_on_error, desc)
            } else {
                view.showErrorWithLogCollection("ERROR", "Cannot continue: " + actions_modal.msg_on_error);
            }
        }
    },

    actionFailed: function(json) {
        actions_modal.setActionResult("error");
        actions_modal.actionsDone();
        if (json === null) {
            return false;
        }
        view.jsonError(json, "ERROR: " + actions_modal.msg_on_error);
        return true;
    },

    nextAction : function() {
        if (actions_modal.actions.length == 0) {
            actions_modal.actionsDone();
            return;
        }

        var selector = actions_modal.actions.shift();
        actions_modal.current_action = $(".action", selector);
        var scroller = $('#actions-table-scroller');
        var pos = $(selector).position().top;
        if (pos < scroller.scrollTop() || pos + 30 > scroller.height()) {
             scroller.stop().scrollTo(selector);
        }
        spinnerFactory.create(actions_modal.current_action, "tiny");

        action_description = $("td", selector).next().text();
        var current_ajax_data = actions_modal.ajax_data;
        if (current_ajax_data == null) current_ajax_data = {};
        current_ajax_data["action_description"] = action_description;

        var ajaxOptions = {type: "POST",
                           dataType: "json",
                           data: current_ajax_data,
                           cache: false,
                           url: actions_modal.perform_action_url};
        view.ajax(ajaxOptions,
                  actions_modal.actionComplete,
                  actions_modal.actionFailed,
                  actions_modal.unspin(actions_modal.current_action));
    },

    calc_height : function() {
        var table_height = $("#actions-table tr").length * 30;
        return 200 + Math.min(table_height, 300);
    }

}
