var auth = {

    call_with_credentials: function(serial, func, finally_callback) {
        // Call the given function after ensuring that we have credentials for the system
        $.ajax({
            url: sources.auth.has_credentials,
            data: {serial: serial},
            dataType: 'json',
            cache: false,
            success: function(json) {
                if (json.error_message) {
                    view.showErrorWithLogCollection("ERROR: Authentication failed", json.error_message);
                    return;
                }
                if (json.return_value == true) {
                    func();
                }
                else {
                    auth.ask_credentials(serial, func);
                }
                finally_callback && finally_callback();
            },
            error: function(response, status, xhr) {
                view.ajaxFail(response, status, xhr);
                finally_callback && finally_callback();
            }
        });
    },

    spin_call_with_credentials: function(serial, func, finallyCallback) {
        var spinFinallyCallback = function(json) {
            $.unblockUI({fadeOut: 0});
            if (finallyCallback) finallyCallback(json);
        };
        spinnerFactory.blockUI("huge");
        return auth.call_with_credentials(serial, func, spinFinallyCallback);
    },

    spin_call_with_credentials_multiple: function(serials, func) {
        // create a recursive call stack: call "call_with_credentials" for every serial and eventually call func
        call_with_credentials_iterator = function() {
            if (serials.length == 0) {
                func();
                return;
            }
            serial = serials.pop();
            auth.spin_call_with_credentials(serial, call_with_credentials_iterator);
        };
        call_with_credentials_iterator();
    },

    update_credentials_label: function(serial, username, is_default) {
        // Update the username display after credentials have changed
        $('#infinibox-' + serial + ' .infinibox-username').text(username);
        if (is_default) {
            // Update the usernames of all the other infiniboxes, as they might have changed
            $('.infinibox.root').each(function() {
                var box = $(this);
                var other_box_serial = box.attr('data-serial');
                if (serial != other_box_serial) {
                    var ajaxOptions = {type: "POST",
                       dataType: "json",
                       data: {serial: other_box_serial},
                       cache: false,
                       url: sources.auth.get_current_username};
                    view.spinAjax(ajaxOptions,
                        function(data) {
                            var username = data.return_value || 'No credentials set';
                            $('.infinibox-username', box).text(username);
                        }
                    );
                }
            });
        }
    },

    ask_credentials: function(serial, func, title) {
        // Ask the user for credentials, and then call the given function
        infinibox_name = $("#infinibox-" + serial).data("name");
        if (!infinibox_name) {
            infinibox_name = "InfiniBox with serial " + serial;
        }
        title = title || "AUTHENTICATION REQUIRED";
        $("#auth-modal-boxname").text(infinibox_name);
        $("#auth-modal-username, #auth-modal-password").val("");
        $("#auth-modal-remember, #auth-modal-default").prop("checked", null);
        $('#auth-error').hide();
        $("#auth-modal").dialog({
            modal: true,
            width: 540,
            height: 390,
            resizable: false,
            title: title,
            buttons: [
                {
                    text: "OK",
                    click: function(event) {
                        var self = this;
                        $('#auth-error').hide();
                        var username = $("#auth-modal-username").val().trim();
                        var password = $("#auth-modal-password").val().trim();
                        if (!username) {
                            $("#auth-error").text('Please enter the username.').fadeIn();
                            $("#auth-modal-username").focus();
                        }
                        else if (!password) {
                            $("#auth-error").text('Please enter the password.').fadeIn();
                            $("#auth-modal-password").focus();
                        }
                        else {
                            spinnerFactory.blockUI("huge");
                            var remember_me = $("#auth-modal-remember").is(":checked");
                            var is_default = $("#auth-modal-default").is(":checked");
                            auth.set_credentials(serial, username, password,
                                                 remember_me,
                                                 is_default,
                                                 function() {
                                                    auth.update_credentials_label(serial, username, is_default);
                                                    $.unblockUI({fadeOut: 0});
                                                    $(self).dialog("close");
                                                    if (func) {
                                                        func();
                                                    }
                                                 },
                                                 function() {
                                                    $.unblockUI({fadeOut: 0});
                                                    $("#auth-error").text('Invalid username or password.').fadeIn();
                                                    $("#auth-modal-username").focus();
                                                 });
                        }
                    }
                },
                {
                    text: "CANCEL",
                    click: function() {
                        $(this).dialog("close");
                    }
                }
            ]
        });
    },

    set_credentials: function(serial, username, password, remember_me, dflt, onsuccess, onfailure) {
        // Send the given credentials to the server, and call onsuccess/onfailure depending on the result
        $.ajax({
            url: sources.auth.set_credentials,
            type: 'POST',
            data: {serial: serial, username: username, password: password, remember_me: remember_me, 'default': dflt},
            dataType: 'json',
            cache: false,
            success: function(json) {
                if (json.return_value == true) {
                    onsuccess();
                }
                else {
                    onfailure();
                }
            },
            error: view.ajaxFail
        });
    },

    reset_credentials: function() {
        show_areyousure("CONFIRM", "Reset credentials?", "", "RESET", function() {
            spinnerFactory.blockUI("huge");
            $.ajax({
                url: sources.auth.reset_credentials,
                type: 'POST',
                data: null,
                dataType: 'json',
                cache: false,
                success: function() {
                    auth.update_credentials_label(0, null, true);
                },
                error: view.ajaxFail,
                complete: function() {
                    $.unblockUI({fadeOut: 0});
                }
            });
        });
    }

}