var create_vol_modal = {

    height: 535,

    capacity_multiplier: 1000,

    setOptions: function(selElement, optionsList, selectedIndex) {
        selElement.empty();
        for (var i = 0; i < optionsList.length; i++) {
            // Create the option
            var text = optionsList[i].name;
            var opt = $('<option/>').attr('value', text).text(text);
            // Check if it should be selected
            if (i == selectedIndex) {
                opt.attr('selected', 'selected');
            }
            // Add any other properties as data attributes
            for (var key in optionsList[i]) {
                if (key != 'name' && optionsList[i].hasOwnProperty(key)) {
                    opt.attr('data-' + key, optionsList[i][key]);
                }
            }
            // Add it to the dropdown
            selElement.append(opt);
        }
    },

    popup_create_new_with_params: function(json) {
        if (json.return_value.pools.length == 0) {
            var username = $('#infinibox-' + json.return_value.box_serial + ' .infinibox-username').text();
            showError("ERROR", "ERROR: There are no pools in the system, or no pools that user " + username + " can manage.");
            return;
        }
        modal_height = create_vol_modal.height;
        if (!(typeof is_veritas === 'undefined') && is_veritas){
             modal_height -= 150;
        }
        $("#vol-modal-serial").val(json.return_value.box_serial);
        $("#vol-modal-vol-name").val(json.return_value.suggested_volume_name);
        $("#vol-modal-mnt-text").val(json.return_value.suggested_mountpoint);
        $("#vol-modal-size-text").val("").trigger("input");
        $("#vol-modal-lun").val("").trigger("input");
        filesystem_index = json.return_value.filesystems.indexOf(json.return_value.selected_filesystem);
        create_vol_modal.setOptions($("#vol-modal-select-fs"), json.return_value.filesystems, filesystem_index);
        create_vol_modal.setOptions($("#vol-modal-select-pool"), json.return_value.pools, 0);
        create_vol_modal.update_pool_capacity();
        $("#create-vol-modal").dialog({
            modal: true,
            width: 540,
            height: modal_height,
            resizable: false,
            buttons: {
                CONTINUE: function() {
                    create_vol_modal.create_new_validate();
                },
                CANCEL: function() {
                    $(this).dialog("close");
                }
            },
            open: function() {
                $("#vol-modal-vol-name").select().focus();
            }
        });
    },

    popup_create_new: function(event) {
        event.preventDefault();
        readiness.with_provisioning_enabled(function() {
            var selElement = $(event.target);
            var serial = selElement.closest(".infinibox.root").data("serial");
            auth.spin_call_with_credentials(serial,
                function() {
                    var data = {serial: serial};
                    var ajaxSource = sources.operations.provision_params;
                    var ajaxOptions = {type: "POST", dataType: "json", data: data,
                                       cache: false, url: ajaxSource};
                    var selInfinibox = selElement.parents('.infinibox.root');
                    create_vol_modal.dataTable = infinibox.getDataTable(selInfinibox);
                    view.spinAjax(ajaxOptions, create_vol_modal.popup_create_new_with_params);
                }
            );
        });
    },

    get_provision_parameters: function() {
        return {serial: $("#vol-modal-serial").val(),
                filesystem: $("#vol-modal-select-fs").val() || "",
                volume_name: $("#vol-modal-vol-name").val(),
                mountpoint: $("#vol-modal-mnt-text").val() || "",
                pool_name: $("#vol-modal-select-pool").val(),
                volume_size: $("#vol-modal-size-text").val() + $("#vol-modal-size-unit").val(),
                thin: $("#vol-modal-thickthin").is(":checked"),
                do_filesystem: $("#vol-modal-create-fs").is(":checked"),
                do_mount: $("#vol-modal-mount-fs").is(":checked"),
                lun: $("#vol-modal-lun").val() ? $("#vol-modal-lun").val() : null
            };
    },

    create_new_popup_actions: function() {
        var data = create_vol_modal.get_provision_parameters();
        var get_action_url = sources.operations.get_provision_tasks;
        var perform_action_url = sources.operations.run_provision_task;
        $("#create-vol-modal").dialog("close");
        actions_modal.show("CREATE VOLUME STEPS",
                           "Volume creation failed",
                           true,
                           get_action_url,
                           perform_action_url,
                           data,
                           function() {operations.reload_table(create_vol_modal.dataTable)},
                           null);
    },

    create_new_validate: function() {
        // Some client side field validations first
        var name = $("#vol-modal-vol-name").val().trim();
        if (!name) {
            $("#vol-modal-vol-name").focus();
            showError("ERROR", "ERROR: Volume name cannot be empty");
            return;
        }
        var size = $("#vol-modal-size-text").val().trim();
        if (!size || isNaN(size)) {
            $("#vol-modal-size-text").focus();
            showError("ERROR", "ERROR: Volume size is invalid");
            return;
        }
        if ($("#vol-modal-lun").length) {
            var lun = $("#vol-modal-lun").val().trim();
            if (lun && isNaN(lun)) {
                $("#vol-modal-lun").focus();
                showError("ERROR", "ERROR: LUN is invalid");
                return;
            }
        }
        if ($("#vol-modal-mount-fs").length){
            var mnt_enabled = $("#vol-modal-mount-fs").is(":checked");
            var mnt_hidden = $(".vol-modal-mount-fs-row").css("display") == "none";
            var mnt = $("#vol-modal-mnt-text").val().trim();
            if (!mnt_hidden && mnt_enabled && !mnt) {
                $("#vol-modal-mnt-text").focus();
                showError("ERROR", "ERROR: Mount path cannot be empty");
                return;
            }
        }
        // Server side validation
        var data = create_vol_modal.get_provision_parameters();
        var ajaxSource = sources.operations.provision_validation;
        var ajaxOptions = {type: "POST", dataType: "json", data: data,
                           cache: false, url: ajaxSource};
        view.spinAjax(ajaxOptions, create_vol_modal.create_new_popup_actions);
    },

    update_toggles: function() {
        var fs_enabled = $("#vol-modal-create-fs").is(":checked");
        var mnt_enabled = $("#vol-modal-mount-fs").is(":checked");
        $("#vol-modal-select-fs").attr("disabled", !fs_enabled);
        $("#vol-modal-mount-fs").attr("disabled", !fs_enabled);
        $("#vol-modal-mnt-text").attr("disabled", !fs_enabled || !mnt_enabled);
        $("#vol-modal-mount-label").css("color", (fs_enabled && mnt_enabled) ? "#222" : "#999");

    },

    update_pool_capacity: function() {
        var opt = $("#vol-modal-select-pool").find(":selected");
        var free = create_vol_modal.format_size(opt.data('free'));
        var size = create_vol_modal.format_size(opt.data('size'));
        $('#vol-modal-pool-capacity').text(free + ' free out of ' + size);
    },

    format_size: function(size, decimal_places) {
        if (size == 0) {
            return 'N / A'
        }
        decimal_places = decimal_places || 2;
        mult = create_vol_modal.capacity_multiplier;
        if (size >= mult * mult * mult * mult) {
            size = size / mult / mult / mult / mult;
            suffix = ' TB';
        }
        else {
            size = size / mult / mult / mult;
            suffix = ' GB';
        }
        var val = parseFloat(size);
        if (isNaN(val))
            return null;
        var p = Math.pow(10, decimal_places);
        val = Math.floor(val * p) / p;
        if (parseInt(val) == val)
            val = Math.floor(val);
        return val + suffix;
    },

    init: function() {
        $(".create_new_popup").click(create_vol_modal.popup_create_new);
        $("#vol-modal-create-fs").change(create_vol_modal.update_toggles);
        $("#vol-modal-mount-fs").change(create_vol_modal.update_toggles);
        $("#vol-modal-select-pool").change(create_vol_modal.update_pool_capacity);
        $("#vol-modal-vol-name").regexRestrict(RegExp("[^a-zA-Z0-9\\+\\^&\'@{}\\$=!()#%.~_ -]", "g"));
        $("#vol-modal-size-text").regexRestrict(RegExp("[^0-9.]", "g"));
        $("#vol-modal-lun").regexRestrict(RegExp("[^0-9]", "g"));
    }

}
