var operations = {

    // Reload the datatable contating the target DOM element, and optionally call the callback afterwards
    reload_parent_table: function(target, callback) {
        var selInfinibox = target.closest(".infinibox.root");
        var dataTable = infinibox.getDataTable(selInfinibox);
        operations.reload_table(dataTable, callback);
    },

    // Reload the given datatable, and optionally call the callback afterwards
    reload_table: function(dataTable, callback, json) {
        var url = sources.volume_table + "&serial=" + infinibox.getSerial(dataTable.closest(".infinibox.root"));
        done_callback = function(data) {
            dataTable.api().clear();
            var actions_template = _.template($('#actions_template').html());
            _.each(data.return_value, function(volume_info) {
                var vol_link = $('<a/>')
                    .attr('href', volume_info.volume_url)
                    .attr('target', '_blank')
                    .text(volume_info.volume_name);
                var mnt_link = $('<a/>')
                    .attr('href', '#')
                    .attr('data-serial', volume_info.serial)
                    .addClass('open_mount_point')
                    .text(volume_info.mount_point);
                var row = {
                    name:    vol_link[0].outerHTML,
                    device:  volume_info.device_name,
                    mount:   volume_info.mount_point ? mnt_link[0].outerHTML : '',
                    size:    create_vol_modal.format_size(volume_info.volume_size),
                    paths:   volume_info.paths,
                    lun:     volume_info.lun,
                    actions: actions_template(volume_info)
                };
                dataTable.api().row.add(row);
            });
            dataTable.api().draw();
            if (callback) callback(json);
        }
        view.spinAjax({dataType: 'json', type: 'POST', url: url}, done_callback);
    },

    popup_set_credentials: function(event) {
        event.preventDefault();
        var selElement = $(event.target);
        var serial = selElement.closest(".infinibox.root").data("serial");
        auth.ask_credentials(serial, null, "SET CREDENTIALS");
    },

    reload: function(event) {
        spinnerFactory.blockUI("huge");
        window.location.reload(true);
    },

    rescan: function(event) {
        var ajaxOptions = {cache: false, url: sources.operations.rescan};
        view.spinAjax(ajaxOptions, function() {
            operations.reload();
        });
    },

    init: function() {
        map_vol_modal.init();
        create_vol_modal.init();
        performance_modal.init();
        $(".set-credentials").click(operations.popup_set_credentials);
        init_actions_menu();
        operations.init_action_listeners();
    },

    init_action_listeners: function() {
        $('body')
            .on('click', '.infinibox.table .open_mount_point', host.open_mount_point)
            .on('click', '.infinibox.table .test-performance', performance_modal.popup)
            .on('click', '.infinibox.table .resize', resize_modal.popup)
            .on('click', '.infinibox.table .rename', rename_modal.popup)
            .on('click', '.infinibox.table .create_fs', create_fs_modal.popup)
            .on('click', '.infinibox.table .mount', mount_modal.popup)
            .on('click', '.infinibox.table .unmap', operations.unmap)
            .on('click', '.infinibox.table .unmount', operations.unmount);
    },

    unmap: function(event) {
        readiness.with_provisioning_enabled(function() {
            var target = $(event.target);
            var callback = function() {
                operations.perform_operation(target, sources.operations['unmap']);
            };
            var volume_name = target.closest('.actions').data('name');
            show_areyousure('CONFIRM', 'Unmap volume ' + volume_name + '?',
                            'This can cause service interruption or data loss.',
                            'UNMAP', callback);
        });
    },

    unmount: function(event) {
        readiness.with_provisioning_enabled(function() {
            var target = $(event.target);
            var callback = function() {
                operations.perform_operation(target, sources.operations['unmount']);
            };
            var volume_name = target.closest('.actions').data('name');
            show_areyousure('CONFIRM', 'Unmount volume ' + volume_name + '?',
                            'This can cause service interruption or data loss.',
                            'UNMOUNT', callback);
        });
    },

    perform_operation: function(selElement, ajaxSource, data, done_callback) {
        var selInfinibox = selElement.closest('.infinibox.root');
        var system_serial = selInfinibox.data('serial');
        auth.spin_call_with_credentials(system_serial, function() {
            data = data || {};
            data.serial = data.serial || selElement.closest('.actions').data("serial");
            var dataTable = infinibox.getDataTable(selInfinibox);
            var ajaxOptions = {type: "POST", dataType: "json", data: data, cache: false, url: ajaxSource};
            view.spinAjax(ajaxOptions, function(json) {operations.reload_table(dataTable, done_callback, json)});
        });
    },

    metadata_sync: function(event) {
        var all_serials = $('.infinibox.root').map(function() { return $(this).data("serial"); }).get();
        auth.spin_call_with_credentials_multiple(all_serials, function() {
            var ajaxOptions = {type: "POST", cache: false, url: sources.operations.update_metadata};
            view.spinAjax(ajaxOptions, function(json) {
                view.show_summary_dialog("DONE", ["Metadata sync complete."]);
            });
        });
    }

}

var host = {
    initDataTable: function() {
        var foreach = function(index, tableInfinibox) {
            var selector = $(tableInfinibox);
            var options = view.getDataTableOptions();
            //options.stateSave = true;
            options.columns = [
                {name: 'name',    data: 'name'},
                {name: 'device',  data: 'device'},
                {name: 'mount',   data: 'mount'},
                {name: 'size',    data: 'size',    width: "60px",  type: "size"},
                {name: 'paths',   data: 'paths',   width: "80px",  type: "numeric"},
                {name: 'lun',     data: 'lun',     width: "50px",  type: "numeric"},
                {name: 'actions', data: 'actions', width: "192px", orderable: false}
            ];
            options.dom = "C" + options.dom;    // ColVis
            options.colVis = {
                align: "right",
                buttonText: " ",
                exclude: [0, 6]
            };
            options.language.zeroRecords = "No Mapped Volumes";
            var table = view.createDataTable(selector, options);
            // hack to prevent zeroRecords from displaying until loading is done
            table.api().row.add({
                name: ' ',
                device: ' ',
                mount: ' ',
                size: ' ',
                paths: ' ',
                lun: ' ',
                actions: ' '
            });
            table.api().draw();
            operations.reload_table(table);
        }
        $(".infinibox.table").each(foreach);
    },

    inventroyLoadedCallback: function(block_infinibox_operations) {
        var callback = function(json) {
            readiness.init();
            $("#inventory").html(json.return_value);
            $(".hidden").hide();
            actions.initToggle();
            host.initDataTable();
            if (!block_infinibox_operations) {
                registration.init();
                operations.init();
            }
            else {
                $(".managed-by-openstack").show();
            };
            if ($(".infinibox.action-toggler").size() == 1)
                view.select(".infinibox.action-toggler").click();
        };
        return callback;
    },

    inventoryAjax: function() {
        var ajaxOptions = {type: "GET", dataType: "json", cache: false, url: sources.inventory};
        view.spinAjax(ajaxOptions, host.inventroyLoadedCallback(is_openstack_cinder_installed));
    },

    getCallbackOnLoad: function(sources) {
        var callback = function() {
            $.ajaxSetup({timeout: 1000*60*10});
            host.inventoryAjax();
        }
        return callback;
    },

    open_mount_point: function(event) {
        var serial = $(event.target).data("serial");
        $.ajax({
            type: "GET",
            url: sources.operations.open_mount_point,
            data: {serial: serial},
            success: function(json) {
                if (json.error_message) {
                    view.showErrorWithLogCollection("ERROR: Cannot open mount point", json.error_message);
                }
            },
            error: function() {
                view.showErrorWithLogCollection("ERROR: Cannot open mount point");
            }
        });
    }

}
