var map_vol_modal = {
    init_map_popup: function() {
        var tableSelector = $("#map-vol-modal-table");
        var options = view.getDataTableOptions();
        options.searching = true;
        options.columns = [{title: "Volume"}];
        options.language = {zeroRecords: "No Volumes", search: "", loadingRecords: " ", processing: " "};
        options.serverSide = true; // pass sSearch and other parameters to the server, and reload ajax when filtering
        options.deferLoading = 0;  // don't make an ajax request until told explicitly
        options.deferRender = true;
        options.paging = true;
        options.pageLength = 7;
        options.infoCallback = function(settings, start, end, max, total, pre) {
            // calculate "Displaying x of y volumes" text
            var total_volumes = max;
            var total_filtered_volumes = total;
            var displayed_volumes = settings.aoData.length;
            var result_string = "Displaying " + displayed_volumes + " of " + total_filtered_volumes;
            if (total_volumes != total_filtered_volumes) {
                result_string += " matching volumes (" + total_volumes  + " volumes total)";
            } else {
                result_string += " volumes";
            }
            return result_string;
        };
        options.info = true;
        var table = view.createDataTable(tableSelector, options);
    },

    map_vol: function(volume_id) {
        var box_serial = $("#map-vol-modal-serial").val();
        var data = {serial: box_serial, volume_id: volume_id};
        operations.perform_operation(
            map_vol_modal.selInfinibox,
            sources.operations.map_existing_volume,
            data,
            function() {
                map_vol_modal.query_mount(data);
            }
        );
    },

    popup_map: function(event) {
        event.preventDefault();
        readiness.with_provisioning_enabled(function() {
            var buttonSelector = $(event.target);
            var box_serial = buttonSelector.closest(".infinibox.root").data("serial");
            map_vol_modal.clear_map_table();
            $("#map-vol-modal-serial").val(box_serial);
            auth.spin_call_with_credentials(box_serial,
                function() {
                    map_vol_modal.update_popup_map();
                    $("#map-vol-modal").dialog({
                        modal: true,
                        width: 540,
                        height: 540,
                        resizable: false,
                        buttons: {
                            MAP: function() {
                                var volume_id = $("input[name=map-modal-volume-id-radio]:checked").val();
                                if (volume_id == undefined) {
                                    showError("ERROR", "Please select a volume to map.");
                                    return;
                                }
                                map_vol_modal.map_vol(volume_id);
                                $(this).dialog('close');
                            },
                            CANCEL: function() {
                                $(this).dialog('close');
                            }
                        }
                    })
                    map_vol_modal.selInfinibox = buttonSelector.parents('.infinibox.root');
                }
            );
        });
    },

    clear_map_table: function() {
        var dataTable = $("#map-vol-modal-table").dataTable();
        var oSettings = dataTable.fnSettings();
        // bAjaxDataGet prevents an ajax call during 'draw' after 'clear' and 'add'
        oSettings.bAjaxDataGet = false;
        dataTable.api().search('');
        dataTable.api().clear();
        dataTable.api().row.add(['']);      // hack to prevent the "No Volumes" label while loading
        dataTable.api().draw();
        oSettings.bAjaxDataGet = true;
    },

    update_popup_map: function(event) {
        var box_serial = $("#map-vol-modal-serial").val();
        var dataTable = $("#map-vol-modal-table").dataTable();
        var show_clones = $("#map-vol-show-clones input").is(":checked");
        var url = sources.box_volumes + "&serial=" + box_serial + "&show_clones=" + show_clones;
        dataTable.api().ajax.url(url).load();
    },

    query_mount: function(data) {
        // send an ajax query about the volume we just mapped - we want to know if it has
        // a filesystem, and the device serial, so we can pop up the "mount volume" dialog if relevant
        var ajaxOptions = {type: "POST", dataType: "json", data: data,
                           cache: false, url: sources.operations.query_mount_after_map};
        view.spinAjax(ajaxOptions, map_vol_modal.query_mount_result, null, null);
    },

    query_mount_result: function(json) {
        if (json.return_value.has_fs) {
            // the volume we mapped has a filesystem - click its "mount volume" button automatically for the user
            $(".actions[data-serial=" + json.return_value.serial + "] .mount").click();
        }
    },

    init: function() {
        map_vol_modal.init_map_popup();
        $(".map_popup").click(map_vol_modal.popup_map);
        $("#map-vol-show-clones input").change(map_vol_modal.update_popup_map);
    }
}