
HBAAPI_SCRIPT = 'from infi.hbaapi.scripts import hbaapi_real; hbaapi_real()'


class FreeSpaceError(Exception):
    pass


def _iter_linux_boot_items():
    import os
    from infi.vendata.powertools.settings.utils import grub
    from infi.logs_collector.collectables import File, Command
    for filepath in (grub.DEFAULT_GRUB_FILE, grub.DEFAULT_GRUB2_FILE, "/proc/cmdline"):
        if os.path.exists(filepath) and os.path.isfile(filepath):
            yield File(filepath)
    if os.path.exists(grub.BOOT_DIRPATH) and os.path.isdir(grub.BOOT_DIRPATH):
        yield Command("sh", ["-c", "ls -latrhR /boot"], prefix="listdir_boot")


def get_cinder_environment():
    from os import environ, path
    env = environ.copy()
    rcfile = path.abspath(path.expanduser("~/keystonerc_admin"))
    if path.exists(rcfile):
        with open(rcfile) as fd:
            for line in fd.xreadlines():
                try:
                    _, keyval = line.split()  # export ort OS_USERNAME=admin
                    key, val = keyval.split('=')
                    env[key] = val
                except:
                    pass
    return env


def linux():
    from infi.vendata.powertools.cli.openstack import is_openstack_cinder_installed
    from infi.logs_collector.collectables import Directory, Command, Script, File
    from infi.logs_collector.items import linux
    from glob import glob
    import os
    MODPROBE_CONF = "/etc/modprobe.conf"
    MODPROBE_D = "/etc/modprobe.d"
    LOTS_OF_DEVICES = 1000
    SYS_DEVICES_PATH = "/sys/devices/pci*"
    num_devices = len(os.listdir('/dev'))
    scsi_commands_timeout = (num_devices/LOTS_OF_DEVICES+1)*60
    sg_recorder_path = os.path.join(os.path.dirname(__file__), "scsi_commands.sh")
    if num_devices < LOTS_OF_DEVICES:
        pci_devices = [Directory(dirname, "^((?!trace|0s|0u).)*$", timeframe_only=False, recursive=True)
                       for dirname in glob(SYS_DEVICES_PATH)]
    else:
        pci_devices = [Command("sh", ["-c", "find {}".format(dirname)]) for dirname in glob(SYS_DEVICES_PATH)]
    modprobe_items = []
    lvm_items = []
    if os.path.exists(MODPROBE_CONF):
        modprobe_items += [File(MODPROBE_CONF)]
    if os.path.exists(MODPROBE_D) and os.path.isdir(MODPROBE_D):
        modprobe_items += [Directory(MODPROBE_D, timeframe_only=False, recursive=True)]
    if os.path.exists("/usr/sbin/lvdisplay"):
        lvm_items = [Command("vgdisplay", ["-v"]), Command("lvdisplay", ["-v", "--all"])]
    packaging_items = []
    openstack_items = []
    if os.path.exists("/bin/rpm"):
        packaging_items.append(Command("rpm", ["-qa"]))
    if os.path.exists("/usr/bin/dpkg"):
        packaging_items.append(Command("dpkg", ["--get-selections"]))
    if is_openstack_cinder_installed():
        cinder_env = get_cinder_environment()
        openstack_items.append(Directory("/var/log/cinder"))
        openstack_items.append(Directory("/etc/cinder"))
        openstack_items.append(Command(["infini-openstack", "volume-backend", "list"]))
        openstack_items.append(Command(["cinder", "list"], env=cinder_env))
        openstack_items.append(Command(["cinder", "type-list"], env=cinder_env))
        openstack_items.append(Command(["cinder", "service-list"], env=cinder_env))
    return [Command("sh", ["-c", "echo show config | multipathd -k"], prefix="multipath_show_config"),
            Command("sh", ["-c", "echo show maps | multipathd -k"], prefix="multipath_show_maps"),
            Command("sh", ["-c", "echo show topology | multipathd -k"], prefix="multipath_show_topology"),
            Command("sh", ["-c", "echo show paths | multipathd -k"], prefix="multipath_show_paths"),
            Command("sh", ["-c", "sg_map -x || true"]),
            Command("fdisk", ["-l"]),
            Command("modinfo", ["qla2xxx"], prefix="modinfo_qla2"),
            Command("modinfo", ["qla4xxx"], prefix="modinfo_qla4"),
            Command("modinfo", ["lpfc"], prefix="modinfo_lpfc"),
            Command("sh", [sg_recorder_path], prefix="scsi_commands", wait_time_in_seconds=scsi_commands_timeout),
            Command("ifconfig", ["-a"]),
            Directory("/etc/", ".*multipath.*|.*tab", timeframe_only=False),
            Directory("/var/log", "infini.*\.log(\.\d+)?$"),
            Directory("/tmp", "infini.*\.log(\.\d+)?$"),
            Directory("/proc", "scsi", recursive=True, timeframe_only=False),
            Directory("/sys/class", "^((?!trace|0s|0u).)*$", timeframe_only=False, recursive=True),
            Directory("/sys/module", "^((?!trace|0s|0u).)*$", timeframe_only=False, recursive=True),
            Directory("/var/tmp/abrt"),
            Directory("/var/spool/abrt"),
            Script(HBAAPI_SCRIPT, prefix='hbaapi'),
            ] + openstack_items + packaging_items + lvm_items + modprobe_items + pci_devices + list(_iter_linux_boot_items()) + linux()

def sunos():
    from infi.vendata.powertools.utils import is_veritas_multipathing_installed
    from infi.logs_collector.collectables import Directory, Command, Script, File
    files = [File("/etc/path_to_inst"),
             File("/etc/name_to_major"),
             File("/etc/driver_aliases"),
             File("/var/log/syslog")]

    listdir_commands = [Command("sh", ["-c", "ls -la /dev"], prefix="dev_listdir"),
                        Command("sh", ["-c", "ls -la /dev/dsk"], prefix="dev_dsk_listdir"),
                        Command("sh", ["-c", "ls -la /dev/rdsk"], prefix="dev_rdsk_listdir"),
                        Command("sh", ["-c", "ls -la /dev/scsi/array_ctrl"], prefix="array_ctrl_listdir")]
    package_commands = [Command("pkginfo", ["-p"]),
                        Command("pkginfo", ["-i"])]

    conf_dirs = [Directory("/etc/driver/drv"),
                 Directory("/kernel/drv")]
    log_dirs = [Directory("/var/log", "infini.*\.log(\.\d+)?$"),
                Directory("/var/pkg"),
                Directory("/tmp", "infini.*\.log(\.\d+)?$")]

    system_commands = [Command("modinfo"),
                       Command("sh", ["-c", 'echo "sd_max_throttle /U\nfcp_offline_delay /U\nfp_offline_ticker /U" |mdb -k'])]
    scsi_commands = [Command("fcinfo", ["lu", "-v"]),
                     Command("fcinfo", ["hba-port"])]
    lun_commands =  [Command("cfgadm", ["-lao", "show_SCSI_LUN"]),
                     Command("sh", "-c format < /dev/null".split(), prefix="format"),
                     Command("kstat", ["-c", "disk"])]
    mpath_commands = [Command("mpathadm", ["list", "lu"])]

    scripts = [Script(HBAAPI_SCRIPT, prefix='hbaapi')]

    all_commands = files + listdir_commands + package_commands + conf_dirs + log_dirs + system_commands + \
                   scsi_commands + lun_commands + mpath_commands + scripts

    if is_veritas_multipathing_installed():
        all_commands += [Command("vxdmpadm", ["list", "dmpnode"]),
                         Command("sh", ["-c", "ls -la /dev/vx/dmp"], prefix="vx_dmp_listdir"),
                         Command("sh", ["-c", "ls -la /dev/vx/rdmp"], prefix="vx_rdmp_listdir")]
    return all_commands

def aix():
    from infi.logs_collector.collectables import Directory, Command, Script

    commands = [Command("sh", ["-c", "ls -la /dev"], prefix="dev_listdir"),
                Command("sh", ["-c", "lsdev -HF 'name status description class subclass connection type'"], prefix="lsdev"),
                Command("sh", ["-c", "lspath -HF 'status name parent path_id connection'"], prefix="lspath"),
                Command("sh", ["-c", "odmget PdAt PdDv CuAt CuDv | grep -ip nfinidat"], prefix="odm_nfinidat"),
                Command("sh", ["-c", "lspv"], prefix="lspv"),
                Command("sh", ["-c", "for dev in `lsvg`; do lsvg -l $dev; done"], prefix="lsvg"),
                Command("sh", ["-c", "for dev in `lsdev -c adapter -F name`; do echo $dev; lsattr -El $dev; done"], prefix="lsattr_adapters"),
                Command("sh", ["-c", "for dev in `lsdev -c disk -F name`; do echo $dev; lsattr -El $dev; done"], prefix="lsattr_disks"),
                # 'lsdev -c dac' will probably not return anything for infinidat, see HOSTDEV-2102
                Command("sh", ["-c", "for dev in `lsdev -c dac -F name`; do echo $dev; lsattr -El $dev; done"], prefix="lsattr_dacs"),
                Command("sh", ["-c", "for dev in `lsdev -c adapter -F name | grep fcs`; do echo $dev; fcstat -e $dev; done"], prefix="fcstat"),
                Command("sh", ["-c", "for dev in `lsdev -c adapter -F name | grep fcs`; do echo $dev; lscfg -vpl $dev; done"], prefix="lscfg_adapters"),
                Command("sh", ["-c", "oslevel -s"], prefix="oslevel"),
                Command("sh", ["-c", "lslpp -l"], prefix="lslpp"),
                Command("sh", ["-c", "errpt"], prefix="errpt")]

    logs = [Directory("/var/log", "infini.*\.log(\.\d+)?$"),
            Directory("/tmp", "infini.*\.log(\.\d+)?$")]

    scripts = [Script(HBAAPI_SCRIPT, prefix='hbaapi')]

    return commands + logs + scripts

def _get_windows_scsi_recorder_command():
    # unlike the linux version, which is simple as it runs for /dev/sg*, there's no easy way to do that on windows
    # so we have two options, we start from one and fail-back to the other:
    # get the device names from infi.devicemanager.DeviceManger().scsi_devices[0].psuedo_device_object
    # use sg_scan to get the device names, problem is that only gives us the volumes, not all scsi devices
    from infi.logs_collector.collectables.windows import CMD_PATH
    from infi.logs_collector.collectables import Command
    import os
    sg_recorder_path = os.path.join(os.path.dirname(__file__), "scsi_commands.cmd")

    def get_scsi_device_names():
        from infi.devicemanager import DeviceManager
        return [device.psuedo_device_object for device in DeviceManager().scsi_devices]

    names = []
    try:
        names = get_scsi_device_names()
    except:
        pass
    return Command(CMD_PATH, ["/c", sg_recorder_path] + names, prefix="scsi_commands")


def windows():
    from infi.logs_collector.collectables import Directory, Command, Script
    from infi.logs_collector.collectables.windows import CMD_PATH, DISKPART_PATH, SYSTEMROOT
    from infi.logs_collector.items import windows
    import os
    return [Command(CMD_PATH, ["/c", "echo LIST DISK | {}".format(DISKPART_PATH)], prefix="list_disk"),
            Command(CMD_PATH, ["/c", "echo LIST VOLUME | {}".format(DISKPART_PATH)], prefix="list_volume"),
            Directory(os.path.join(SYSTEMROOT, "Temp"), regex_basename=".*infi_vss.*"),
            Script(HBAAPI_SCRIPT, prefix='hbaapi'),
            _get_windows_scsi_recorder_command()] + windows()


def get_powertools_commands():
    from infi.logs_collector.collectables import Command
    return [
            Command("infinihost", ["-v"], prefix="infinihost_version"),
            Command("infinihost", ["connectivity", "list"], prefix="infinihost_connectivity_list"),
            Command("infinihost", ["connectivity", "list", "--json"], prefix="infinihost_connectivity_list__json"),
            Command("infinihost", ["settings", "check"], prefix="infinihost_settings_check"),
            Command("infinihost", ["volume", "list"], prefix="infinihost_volume_list"),
            Command("infinihost", ["volume", "list", "--json"], prefix="infinihost_volume_list__json"),
            ]


def get_powertools_logs():
    from infi.vendata.powertools.utils.logging import LogFileHandler
    from infi.logs_collector.collectables import Directory
    return [Directory(LogFileHandler.get_logs_directory(), 'infi.*\.log(\.\d+)?$')]


def get_powertools_items():
    return get_powertools_commands() + get_powertools_logs()


def get_items():
    from ..utils import get_platform_name
    platform_name = get_platform_name()
    platform_func = globals().get(platform_name, list)
    return platform_func() + get_powertools_items()


def run(timestamp, delta, optional_archive_path=None, silent=False, interactive=False, only_infinihost_logs=False):
    from infi.logs_collector.scripts import parse_datestring, parse_deltastring
    from infi.storagemodel import get_storage_model
    from tempfile import gettempdir
    timestamp = parse_datestring(timestamp)
    delta = parse_deltastring(delta)

    REQUIRED_FREE_SPACE_GB = 1.0
    free_space_gb = get_storage_model().get_utils().get_free_space(gettempdir()) / 1024.0 / 1024 / 1024
    if free_space_gb < REQUIRED_FREE_SPACE_GB:
        msg = "Not enough free space for log collection. Available:{:.3f} GB, Required: {:.3f} GB"
        print msg.format(free_space_gb, REQUIRED_FREE_SPACE_GB)
        raise FreeSpaceError(msg.format(free_space_gb, REQUIRED_FREE_SPACE_GB))

    from infi.logs_collector import run
    items = get_powertools_logs() if only_infinihost_logs else get_items()
    return run("infinihost", items, timestamp, delta, optional_archive_path,
               parent_dir_name="infinihost-diagnostics", silent=silent, interactive=interactive)
