from infi.vendata.powertools.utils.versioncheck import UnsupportedVersion
from ..system_register.engine import update_metadata, output_stream_context, update_metadata_on_controller, Abort
from infi.pyutils.lazy import cached_function, clear_cache
from logging import getLogger
logger = getLogger(__name__)


@cached_function
def get_credentials(serial, silent):
    from ..utils import auth
    # in silent mode, we skip systems that don't have credentials
    # GUI sync button also uses silent mode and will ensure there are credentials for all systems
    if silent:
        return auth.CREDENTIALS_STORE.get_existing_credentials(serial)
    return auth.CREDENTIALS_STORE.get_credentials(serial)

def get_device_filesystem_name(device):
    from ..utils import safe_get_device_partitions
    partitions = safe_get_device_partitions(device)
    if len(partitions) != 1:
        return None
    return partitions[0].get_current_filesystem().get_name()

def volume_metadata_sync(silent):
    from ..utils import get_mount_point
    from ..utils.queries import get_infinisdk_host_and_volume_by_device
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices
    from ..utils.versioncheck import is_supported
    for device in get_infinidat_block_devices():
        serial = device.get_vendor().get_system_serial()
        if not is_supported(device.get_vendor().get_system_version()):
            continue
        if get_credentials(serial, silent):
            try:
                _, volume = get_infinisdk_host_and_volume_by_device(device)

                mountpoint = get_mount_point(device)
                if mountpoint is None:  # filesystem not mounted
                    continue
                filesystem = get_device_filesystem_name(device)
                update_metadata(volume, mount_point=mountpoint, filesystem_type=filesystem)
            except (Abort, UnsupportedVersion):
                pass

def host_metadata_sync(silent):
    from ..utils.queries import get_uniq_list_of_attached_boxes__by_serial
    from ..utils.versioncheck import is_supported
    for controller in get_uniq_list_of_attached_boxes__by_serial():
        serial = controller.get_vendor().get_system_serial()
        if not is_supported(controller.get_vendor().get_system_version()):
            continue
        if get_credentials(serial, silent):
            try:
                update_metadata_on_controller(controller)
            except (Abort, UnsupportedVersion):
                pass

def metadata_sync(silent=False, raise_on_error=True):
    from StringIO import StringIO
    stream = StringIO() if silent else None
    clear_cache(get_credentials)
    with output_stream_context(stream):
        try:
            host_metadata_sync(silent)
            volume_metadata_sync(silent)
        except:
            if raise_on_error:
                raise
            logger.exception("silent metadata sync failed")
