import sys
from ..utils.progress_bar import progress_bar_context
from ..utils.queries import get_system_object_by_serial, get_pools
from ..utils.queries import get_uniq_list_of_attached_boxes__by_serial
from ..utils.tables import Table, Column
from ..utils.termsize import getTerminalSize
from ..rescan.engine import do_rescan
import capacity

def _build_inventory(system_serial):
    systems = []
    serials = []
    boxes = get_uniq_list_of_attached_boxes__by_serial()
    for box in boxes:
        serials.append(box.get_vendor().get_system_serial())
    for serial in sorted(serials):
        if system_serial is None or system_serial == serial:
            system = get_system_object_by_serial(serial)
            systems.append(dict(system=system, pools=get_pools(system)))
    return systems


def _build_table(system_serial, size_units):

    def size_formatter(size_in_bytes):
        size_unit = getattr(capacity, size_units)
        size_unit_in_bytes = size_unit / capacity.byte
        return "{:.2f}".format(float(size_in_bytes) / float(size_unit_in_bytes))

    systems = _build_inventory(system_serial)
    table = Table([Column(0, 'System'),
                   Column(1, 'Pool'),
                   Column(2, 'Physical Capacity (%s)' % size_units, formatter=size_formatter, right_aligned=True),
                   Column(3, 'Virtual Capacity (%s)' % size_units, formatter=size_formatter, right_aligned=True),
                   Column(4, 'Volumes', right_aligned=True),
                  ], max_width=getTerminalSize()[0])
    for item in systems:
        for pool in item['pools']:
            table.add_row([item['system'].get_name(),
                           pool.get_name(),
                           pool.get_physical_capacity() // capacity.byte,
                           pool.get_virtual_capacity() // capacity.byte,
                           pool.get_field('volumes_count')])
    return table


def run(system_serial, size_units='GB', rescan=True):
    if rescan:
        sys.stderr.write("\rRescanning... ")
        sys.stderr.flush()
        do_rescan()
    sys.stderr.write("\rQuerying pools... ")
    sys.stderr.flush()
    with progress_bar_context():
        table = _build_table(system_serial, size_units)
    sys.stderr.write("\r" + " " * (len("Querying pools... ") + 1) + "\r")
    sys.stderr.flush()
    if len(table.rows):
        print table
    else:
        print "No pools detected."
