from ..utils.progress_bar import progress_bar_context


def test_all_infinidat_non_multipathed_block_devices_are_connected():
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_non_multipathed_scsi_block_devices
    devices = get_infinidat_non_multipathed_scsi_block_devices()
    for device in devices:
        if device.get_scsi_standard_inquiry().peripheral_device.qualifier != 0:  # (not connected)
            return False
    return True


def test_all_infinidat_multipathed_block_devices_are_connected():
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_native_multipath_block_devices
    devices = get_infinidat_native_multipath_block_devices()
    for device in devices:
        if device.get_scsi_standard_inquiry().peripheral_device.qualifier != 0:  # (not connected)
            return False
    return True


def test_unit_ready_for_all_infinidat_devices():
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices_and_controllers
    [device.get_scsi_test_unit_ready() for device in get_infinidat_block_devices_and_controllers()]
    return True


def are_there_single_path_block_devices_from_box(box):
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_scsi_block_devices
    return bool([device for device in get_infinidat_scsi_block_devices()
                if device.get_vendor().get_system_serial() == box])


def are_there_multi_path_block_devices_from_box(box):
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_native_multipath_block_devices
    return bool([device for device in get_infinidat_native_multipath_block_devices()
                if device.get_vendor().get_system_serial() == box])


def reconfigure_multipathd():
    from infi.multipathtools import MultipathClient
    from infi.multipathtools.errors import ConnectionError
    from logging import getLogger
    logger = getLogger(__name__)
    try:
        MultipathClient().rescan()
    except ConnectionError:
        logger.exception("Caught an error while reconfiguring multipathd")


def _is_linux_storagemodel():
    try:
        from infi.storagemodel import get_storage_model
        from infi.storagemodel.linux import LinuxStorageModel
        return isinstance(get_storage_model(), LinuxStorageModel)
    except:
        return False


def consider_reconfigure_multipathd():
    if _is_linux_storagemodel():
        reconfigure_multipathd()


def do_rescan(disable_print=False, wait_for_completion=True):
    from infi.storagemodel import get_storage_model
    from infi.storagemodel.predicates import PredicateList
    get_storage_model().rescan_and_wait_for(wait_for_completion=wait_for_completion)
    predicates = PredicateList([test_unit_ready_for_all_infinidat_devices,
                                test_all_infinidat_multipathed_block_devices_are_connected,
                                test_all_infinidat_non_multipathed_block_devices_are_connected])
    with progress_bar_context(silent=disable_print):
        get_storage_model().rescan_and_wait_for(predicates)
        consider_reconfigure_multipathd()


def clear_unit_attention_from_infinidat_storage_controllers():
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices_and_controllers__mapped_to_lun0
    from infi.storagemodel.errors import DeviceDisappeared, RescanIsNeeded, StorageModelFindError
    devices = []

    try:
        devices.extend(get_infinidat_block_devices_and_controllers__mapped_to_lun0())
    except RescanIsNeeded:
        pass

    for device in devices:
        while True:
            try:
                device.get_scsi_test_unit_ready()
                break
            except DeviceDisappeared:
                break
            except (RescanIsNeeded, StorageModelFindError):
                continue
