from .....case import FixableCase
from infi.vendata.powertools.utils import execute_command

# pylint: disable=R0904,W0201

class KernelModule(object):
    def __init__(self, module_name):
        self._name = module_name

    def is_loaded(self):
        stdout = execute_command("/sbin/lsmod".split()).get_stdout()
        return self._name in stdout

    def load(self):
        execute_command("modprobe {}".format(self._name).split())

    def unload(self):
        execute_command("rmmod {}".format(self._name).split())

    def __repr__(self):
        return "<Kernelmodule: {}>".format(self._name)


class KernelModuleLoaded(FixableCase):
    def __init__(self, category, kb_number, module_name):
        super(KernelModuleLoaded, self).__init__(category)
        self._kb_number = kb_number
        self._module_name = module_name

    def run_test(self):
        self._kernel_module = KernelModule(self.get_kernel_module_name())
        self.assertTrue(self._kernel_module.is_loaded(),
                        "module {} is not loaded".format(self.get_kernel_module_name()))

    def get_fix_description(self):
        return "load kernel module {}".format(self.get_kernel_module_name())

    def run_fix(self):
        self._kernel_module.load()

    def get_description(self):
        return "Checking that kernel module {} is loaded.".format(self.get_kernel_module_name())

    def get_kb_number(self):
        return self._kb_number

    def get_kernel_module_name(self):
        return self._module_name
