from .....case import FixableCase

# pylint: disable=R0904,W0201

class ServiceBaseCase(FixableCase):
    def __init__(self, category, kb_number, service_name, process_name, init_service_class):
        super(ServiceBaseCase, self).__init__(category)
        self._kb_number = kb_number
        self._service_name = service_name
        self._process_name = process_name
        self._init_service_class = init_service_class

    def get_kb_number(self):
        return self._kb_number

    def get_service_name(self):
        return self._service_name

    def get_process_name(self):
        return self._process_name

    def get_init_service(self):
        return self._init_service_class(self.get_service_name(), self.get_process_name())

class ServiceAutoLoadTask(ServiceBaseCase):
    def run_test(self):
        self._service = self.get_init_service()
        self.assertTrue(self._service.is_auto_start(),
                        "service {} is not set to auto-start".format(self.get_service_name()))

    def get_fix_description(self):
        return "configure service {} to automatically load on boot".format(self.get_service_name())

    def run_fix(self):
        self._service.set_auto_start()

    def get_description(self):
        return "Checking that {} automatically loads on boot".format(self.get_service_name())

class ServiceStartedTask(ServiceBaseCase):
    def run_test(self):
        self._service = self.get_init_service()
        self.assertTrue(self._service.is_running(), "service {} is not running".format(self.get_service_name()))

    def get_fix_description(self):
        return "start service {}".format(self.get_service_name())

    def run_fix(self):
        self._service.start()

    def get_description(self):
        return "Checking that {} is running.".format(self.get_service_name())
