from infi.os_info import get_platform_string
from os import path
import re

SOLARIS_NEW_DRIVERS_PATH = '/etc/driver/drv'
SOLARIS_OLD_DRIVERS_PATH = '/kernel/drv'

class SolarisDriverPropertiesMixin(object):
    def _get_config_filename(self, driver_name):
        # in solaris 11 the relevant configuration file has been moved into the etc folder
        os_ver = get_platform_string().split("-")[1]
        if os_ver >= 11 and path.exists(path.join(SOLARIS_NEW_DRIVERS_PATH, driver_name + '.conf')):
            return path.join(SOLARIS_NEW_DRIVERS_PATH, driver_name + '.conf')
        else:
            return path.join(SOLARIS_OLD_DRIVERS_PATH, driver_name + '.conf')

    def _replace_key_value_in_configuration(self, config_data, config_key, new_value):
        # support only keyname="value" keys
        found_key = re.search("^\s*{}\s*=\s*([\w\-\"/\,\s\n_\#]*)\s*;\s*$".format(config_key), config_data, re.MULTILINE)
        key_data = "{}={};\n".format(config_key, new_value)
        if found_key:
            config_data = ''.join([config_data[:found_key.start()], key_data, config_data[found_key.end():]])
        else:
            config_data = '\n'.join([config_data, key_data])
        return config_data

    def _get_current_config(self, driver_name):
        with open(self._get_config_filename(driver_name), "r") as fd:
            header_regex = re.compile('^\s*(name?\s*=\s*[\w\s\=\"\-]*\s*;)$', re.MULTILINE)
            attributes_regex = re.compile('^\s*([\w\-\_]*)?\s*=\s*([\w\-\"/\,\s\n_\#]*)\s*;$', re.MULTILINE)
            data = fd.read()
            return header_regex.findall(data), dict(attributes_regex.findall(data))
