from ....case import FixableCase, Category
from ..common import SolarisDriverPropertiesMixin
from infi.execute import execute_assert_success
from os.path import exists

# pylint: disable=R0904,W0201


class FiberChannelDriverProperties(FixableCase, SolarisDriverPropertiesMixin):
    def __init__(self, *args, **kwargs):
        super(FiberChannelDriverProperties, self).__init__(*args, **kwargs)

    def _backup_current_config(self, driver_name):
        filename = self._get_config_filename(driver_name)
        data = open(filename).read()
        open(filename + '.bak', 'w').write(data)

    def _get_case_config(self):
        from infi.vendata.powertools.conf import get_configuration
        return get_configuration().get('readiness.cases.solaris_fc_driver_attributes')

    def run_test(self):
        from re import match
        config = self._get_case_config()
        for driver_name in config.keys():
            if not exists(self._get_config_filename(driver_name)):
                continue
            headers, current_config = self._get_current_config(driver_name)
            for key, expected in config[driver_name].items():
                msg = "Property {} of {} driver is {}, when it should be {}"
                actual = current_config.get(key, "non-existent")
                equals = bool(match(actual, expected))
                self.assertTrue(equals, msg.format(key, driver_name, actual, expected))

    def _write_driver_parameters(self):
        from re import match
        config = self._get_case_config()
        for driver_name in config.keys():
            self._backup_current_config(driver_name)
            headers, current_config = self._get_current_config(driver_name)
            if not current_config:
                continue
            config_data = open(self._get_config_filename(driver_name), "r").read()
            for key, expected in config[driver_name].items():
                actual = current_config.get(key, "non-existent")
                equals = bool(match(actual, expected))
                if not equals:
                    config_data = self._replace_key_value_in_configuration(config_data, key, expected)
            with open(self._get_config_filename(driver_name), 'w') as fd:
                fd.write(config_data)

    def get_fix_description(self):
        return "Override the fiberchannel driver attributes"

    def run_fix(self):
        # TODO backup boot environment with beadm?
        self._write_driver_parameters()
        execute_assert_success(["bootadm", "update-archive", "-v"])
        self.ask_for_reboot("Reboot is required for the changes to take effect")

    def get_description(self):
        return "Checking FC HBA driver attributes"

    def get_kb_number(self):
        return "202403811"


def get_category():
    return Category("FC Driver Attributes", [FiberChannelDriverProperties, ])
