from infi.vendata.powertools.utils import is_veritas_multipathing_installed
from ..common import SolarisDriverPropertiesMixin
from ....case import FixableCase, Category
from os.path import exists

class MultipathEnabled(FixableCase, SolarisDriverPropertiesMixin):
    def __init__(self, *args, **kwargs):
        super(MultipathEnabled, self).__init__(*args, **kwargs)

    def _get_devices_to_enable(self):
        return ["fp", "mpt"]

    def _get_case_config(self):
        from infi.vendata.powertools.conf import get_configuration
        return get_configuration().get('readiness.cases.solaris_fc_driver_attributes')

    def _is_multipath_enabled(self, config):
        if config.get("mpxio-disable").strip("\'\"") == "yes":
            return False
        else:
            return True

    def _enable_multipath(self, device):
        from pexpect import spawn
        p = spawn("stmsboot -D {} -e".format(device))
        index = p.expect([".*This operation will require a reboot.*Do you want to continue ?.*",
                          "STMS is already enabled. No changes or reboots needed.*"])
        if index == 1:
            p.terminate()  # just to make sure
            return
        p.sendline("y")
        p.expect(".*Reboot the system now ?.*")
        p.sendline("n")
        p.terminate()  # just to make sure

    def run_test(self):
        for device_name in self._get_devices_to_enable():
            if not exists(self._get_config_filename(device_name)):
                self.warn("Can not find {} configuration".format(device_name))
                continue
            headers, current_config = self._get_current_config(device_name)
            self.assertTrue(self._is_multipath_enabled(current_config), "Multipath is disabled")

    def get_fix_description(self):
        return "Enable mpxio for devices"

    def run_fix(self):
        for device_name in self._get_devices_to_enable():
            self._enable_multipath(device_name)
        self.ask_for_reboot("Reboot is required for the changes to take effect")

    def get_description(self):
        return "Verifying that multipath is enabled"

    def get_kb_number(self):
        return "203081601"


def get_category():
    if not is_veritas_multipathing_installed():
        return Category("Multipath", [MultipathEnabled, ])
    else:
        return Category("Multipath", [])
