
import os

SYSTEM_ROOT = os.environ.get('SystemRoot', r'C:\Windows')

OCSETUP_PATH = os.path.join(SYSTEM_ROOT, 'System32', 'ocsetup.exe')
OCSETUP_PARAMETERS = ['MultipathIo', '/norestart', '/quiet']

DISM_PATH = os.path.join(SYSTEM_ROOT, 'System32', 'Dism.exe')
DISM_PARAMETERS = ['/Online', '/Enable-Feature=MultipathIo', '/NoRestart', '/Quiet']

SUCCESS = 0
REBOOT = 3010

class RebootRequired(Exception):
    pass

from infi.wmi import WmiClient

class MultipathIoFeature(object):
    def is_installed(self):
        for _ in WmiClient().execute_query("SELECT * FROM Win32_ServerFeature WHERE ID=57"):
            return True
        return False

    def are_there_infinidat_block_devices(self):
        from infi.pyutils.lazy import clear_cache
        from infi.storagemodel import get_storage_model
        from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices
        clear_cache(get_storage_model())
        try:
            return len(get_infinidat_block_devices()) > 0
        except:
            return True

    def install(self):
        if os.path.exists(OCSETUP_PATH):
            self._install(OCSETUP_PATH, OCSETUP_PARAMETERS)
        else:
            self._install(DISM_PATH, DISM_PARAMETERS)

    def _install(self, path, parameters):
        from infi.execute import execute, ExecutionError
        cmd = execute([path] + parameters)
        cmd.wait()
        rc = cmd.get_returncode()
        if rc == REBOOT or self.are_there_infinidat_block_devices():
            raise RebootRequired()
        if rc == SUCCESS:
            return
        raise ExecutionError(cmd)
