
from infi.pyutils.contexts import contextmanager

@contextmanager
def with_StringIO():  # pylint: disable=C0103
    from StringIO import StringIO
    stream = StringIO()
    try:
        yield stream
    finally:
        stream.close()

def _open_grub_config():
    from .grub import GrubConfig
    from infi.os_info import get_platform_string
    is_suse_11 = 'suse-11' in get_platform_string()
    return GrubConfig(filepath='/boot/grub/menu.lst') if is_suse_11 else GrubConfig()

def create_backup_entry_in_grub(initrd, backup_initrd, description=None):
    from .grub import GrubBootEntry
    grub = _open_grub_config()
    grub.create_backup()
    for entry in filter(lambda entry: entry.get_initrd_image() == initrd, grub.get_boot_entries()):
        new_entry = GrubBootEntry.from_grub_conf(str(entry))
        title = "[INFINIDAT Backup] {}".format(description or entry.get_title())
        new_entry.set_title(title)
        new_entry.set_initrd_image(backup_initrd)
        grub.add_boot_entry(new_entry)
        return

def backup_and_create_initrd(description=None):
    from .initrd import create_initrd
    from datetime import datetime
    initrd = _open_grub_config().get_default_boot_entry().get_initrd_image()
    backup_initrd = create_initrd(initrd)
    if description:
        description = "{}, {}".format(description, datetime.now().ctime())
    create_backup_entry_in_grub(initrd, backup_initrd, description)
