import os
from infi import unittest
from .. import GrubConfig
import mock

EXAMPLES_DIR = os.path.dirname(__file__)
EXAMPLES = [os.path.join(EXAMPLES_DIR, "example1"), os.path.join(EXAMPLES_DIR, "example2"),
            os.path.join(EXAMPLES_DIR, "example3")]


class ExampleTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super(ExampleTestCase, cls).setUpClass()
        from .. import Grub1Config
        if os.name == "nt":
            raise unittest.SkipTest("skipping on Windows")
        if GrubConfig is not Grub1Config:
            raise unittest.SkipTest("Unit tests for Grub 1 skipped in Grub2 environment")

    def test__1(self):
        grub = GrubConfig(EXAMPLES[0])
        entries = grub.get_boot_entries()
        self.assertEqual(len(entries), 1)
        entry = entries.pop()
        self.assertEqual(entry.get_title(), "Red Hat Enterprise Linux Server (2.6.18-229.el5)")
        self.assertEqual(entry.get_root_device(), "(hd0,0)")
        self.assertEqual(str(entry), """title Red Hat Enterprise Linux Server (2.6.18-229.el5)
\troot (hd0,0)
\tkernel /boot/vmlinuz-2.6.18-229.el5 ro root=LABEL=/
\tinitrd /boot/initrd-2.6.18-229.el5.img
""")

    def test__2(self):
        grub = GrubConfig(EXAMPLES[1])
        entries = grub.get_boot_entries()
        self.assertEqual(len(entries), 5)

    def test__3(self):
        with mock.patch("os.path.ismount") as ismount:
            ismount.return_value = True
            grub = GrubConfig(EXAMPLES[2])
            entries = grub.get_boot_entries()
            self.assertGreater(len(entries), 1)
            self.assertFalse(any(i.get_initrd_image() is None for i in entries))
            self.assertFalse(any(i.get_title() is None for i in entries))
            self.assertEqual(str(entries[0]), """title Red Hat Enterprise Linux Server (2.6.18-194.3.1.el5)
root (hd0,0)
cat /message.storix
kernel /vmlinuz-2.6.18-194.3.1.el5 root=/dev/rootvg/rootlv rhgb quiet console=tty0 selinux=0 crashkernel=128M@16M
initrd /initrd-2.6.18-194.3.1.el5.img
""")
            self.assertEquals(entries[0].get_initrd_image(), "/boot/initrd-2.6.18-194.3.1.el5.img")
            self.assertEquals(entries[0], grub.get_default_boot_entry())
            entries[0].set_title("x")
            entries[0].set_initrd_image("/boot/y")
            self.assertEqual(str(entries[0]), """title x
root (hd0,0)
cat /message.storix
kernel /vmlinuz-2.6.18-194.3.1.el5 root=/dev/rootvg/rootlv rhgb quiet console=tty0 selinux=0 crashkernel=128M@16M
initrd /y
""")

    def test_remove_append(self):
        grub = GrubConfig(EXAMPLES[0])
        entries = grub.get_boot_entries()
        self.assertEqual(len(entries), 1)
        entry = entries.pop()
        grub.remove_boot_entry(entry)
        entries = grub.get_boot_entries()
        self.assertEqual(len(entries), 0)
        grub.add_boot_entry(entry)
        entries = grub.get_boot_entries()
        self.assertEqual(len(entries), 1)

    def test_get_default(self):
        grub = GrubConfig(EXAMPLES[0])
        entries = grub.get_boot_entries()
        self.assertEqual(len(entries), 1)
        entry = entries.pop()
        self.assertEqual(entry, grub.get_default_boot_entry())
