from logging import getLogger
import platform

logger = getLogger(__name__)

KERNEL_VERSION = platform.uname()[2]

def create_initrd(filepath, create_backup=True, kernel_version=KERNEL_VERSION):
    """:returns: the path of the backup image, if exists, else None"""
    from shutil import copyfile
    from os.path import exists, dirname, basename
    from os import close
    from tempfile import mkstemp
    from ....utils import execute_command
    basedir, filename = dirname(filepath), basename(filepath)
    backup_filepath = None
    if create_backup:
        if not exists(filepath):
            raise RuntimeError("Cannot backup initrd {} because it does not exist".format(filepath))
        fd, path = mkstemp(prefix=filename, dir=basedir)
        close(fd)
        copyfile(filepath, path)
        backup_filepath = path
    if "suse" in platform.linux_distribution()[0].lower():
        execute_command(['mkinitrd'])
    elif exists("/usr/sbin/update-initramfs"):
        # we could use "mkinitramfs -o [filepath] [kernel_version]",
        # but it doesn't update the modprobe configuration for some reason
        execute_command(['update-initramfs', '-u'])
    elif exists("/sbin/dracut"):
        execute_command(['dracut', '-f', filepath])
    else:
        execute_command(['mkinitrd', '-f', filepath, kernel_version])
    return backup_filepath
