
from logging import getLogger
log = getLogger(__name__)

def remove_port_from_host(host_object, port_address):
    host_object.remove_fc_port(port_address)

def create_host(system_object, host_name):
    log.info("Creating host with name {!r} on system {!r}".format(host_name,
                                                                  system_object.get_name()))
    host = system_object.hosts.create(name=host_name)
    log.info("Host with name {!r} and id {!r} was created".format(host.get_name(),
                                                                  host.get_id()))
    return host

def add_port_to_host(host_object, port_address):
    log.info("Adding port {!r} to host id {!r}".format(port_address, host_object.get_id()))
    host_object.add_fc_port(port_address)
    log.info("Host id {!r} now contains the following ports: {!r}".format(host_object.get_id(),
                                                                          host_object.get_fc_ports()))
def add_ports_to_host(host_object, port_addresses):
    for port_address in port_addresses:
        add_port_to_host(host_object, port_address)

def delete_host_from_box(host_object):
    cluster = host_object.get_cluster(from_cache=False)
    if cluster is not None:
        cluster.remove_host(host_object)
    for lun in list(host_object.get_luns(from_cache=False)):
        host_object.unmap_volume(lun=lun)
    host_object.delete()

def get_existing_host(system_object, host_name):
    from infinisdk.core.exceptions import ObjectNotFound
    try:
        return system_object.hosts.get(name=host_name)
    except ObjectNotFound:
        return None
