import sys
import errno
import logging
from infi.pyutils.decorators import wraps


def no_logging():
    return len(logging.root.handlers) == 0


def logging_to_stream(stream):
    return any(isinstance(handler, logging.StreamHandler) and handler.stream is stream for
               handler in logging.root.handler)


class StreamWrapper(object):
    def __init__(self, fileobj):
        super(StreamWrapper, self).__init__()
        self._fileobj = fileobj
        self.logger = logging.getLogger(str(fileobj.name))

    def __getattr__(self, name):
        try:
            result = getattr(self._fileobj, name)
            return silence_broken_pipe_decorator(result) if callable(result) else result
        except:
            super(StreamWrapper, self).__getattr__(name)

    def __getitem__(self, name):
        try:
            result = getattr(self._fileobj, name)
            return silence_broken_pipe_decorator(result) if callable(result) else result
        except:
            super(StreamWrapper, self).__getitem__(name)

    def write(self, *args, **kwargs):
        try:
            value = args[0].strip()
            if no_logging() or not logging_to_stream(self._fileobj):
                return
            if value:
                self.logger.debug(value)
        except:
            pass
        finally:
            self.__getattr__('write')(*args, **kwargs)

def silence_broken_pipe_decorator(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except IOError as error:
            if error.errno != errno.EPIPE:
                raise
    return wrapper


def silence_broken_pipe_on_stdout_and_stderr():
    if 'nosetests' in sys.argv[0]:
        return
    import colorama.initialise
    sys.stdout = StreamWrapper(sys.__stdout__)
    sys.stderr = StreamWrapper(sys.__stderr__)
    colorama.initialise.orig_stdout = sys.stdout
    colorama.initialise.orig_stderr = sys.stderr
