import sys

from mock import patch, MagicMock
from infi.pyutils.contexts import contextmanager
from pyfakefs.fake_filesystem_unittest import TestCase as FakeFSTestCase
from pyfakefs.fake_filesystem import FakeOsModule, FakeFileOpen
from infi.vendata.powertools.settings import case
from infi.vendata.powertools.settings.cases.linux.base_cases import device_mapper
from infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper import powerpath


GLOBAL_MULTIPATH_CONF_PATH = '/etc/multipath.conf'
RECOMMENDED_MULTIPATH_CONF_CONTENT = open(device_mapper.MULTIPATH_CONF_PATH).read()

INFINIDAT_WWIDS = [u'742b0f000004e4f00000000000001cb',
 u'742b0f000004e4f0000000000000ace',
 u'742b0f000004e4f0000000000000ae0',
 u'742b0f000004e4f0000000000000aec',
 u'742b0f000004e4f0000000000000aee',
 u'742b0f000004e4f0000000000000afc',
 u'742b0f000004e4f0000000000000aff',
 u'742b0f000004e4f0000000000000b02',
 u'742b0f000004e4f0000000000000b05',
 u'742b0f000004e4f00000000000001cb',
 u'742b0f000004e4f0000000000000ace',
 u'742b0f000004e4f0000000000000ae0',
 u'742b0f000004e4f0000000000000aec',
 u'742b0f000004e4f0000000000000aee',
 u'742b0f000004e4f0000000000000afc',
 u'742b0f000004e4f0000000000000aff',
 u'742b0f000004e4f0000000000000b02',
 u'742b0f000004e4f0000000000000b05',
 u'742b0f000004e4f00000000000001cb',
 u'742b0f000004e4f0000000000000ace',
 u'742b0f000004e4f0000000000000ae0',
 u'742b0f000004e4f0000000000000aec',
 u'742b0f000004e4f0000000000000aee',
 u'742b0f000004e4f0000000000000afc',
 u'742b0f000004e4f0000000000000aff',
 u'742b0f000004e4f0000000000000b02',
 u'742b0f000004e4f0000000000000b05',
 u'742b0f000004e4f0000000000000b08',
 u'742b0f000004e4f0000000000000b0b',
 u'742b0f000004e4f00000000000012d2',
 u'742b0f000004e4f0000000000000b08',
 u'742b0f000004e4f0000000000000b0b',
 u'742b0f000004e4f00000000000012d2',
 u'742b0f000004e4f0000000000000b08',
 u'742b0f000004e4f0000000000000b0b',
 u'742b0f000004e4f00000000000012d2']

MPATH_CONFIG_FIXED = 'defaults {\n\tverbosity 2\n\tpolling_interval 5\n\tmax_polling_interval 20\n\treassign_maps "yes"\n\tmultipath_dir "/lib64/multipath"\n\tpath_selector "service-time 0"\n\tpath_grouping_policy "failover"\n\tuid_attribute "ID_SERIAL"\n\tprio "const"\n\tprio_args ""\n\tfeatures "0"\n\tpath_checker "directio"\n\talias_prefix "mpath"\n\tfailback "manual"\n\trr_min_io 1000\n\trr_min_io_rq 1\n\tmax_fds 8192\n\trr_weight "uniform"\n\tqueue_without_daemon "no"\n\tpg_timeout "none"\n\tflush_on_last_del "no"\n\tuser_friendly_names "yes"\n\tfast_io_fail_tmo 5\n\tbindings_file "/etc/multipath/bindings"\n\twwids_file /etc/multipath/wwids\n\tlog_checker_err always\n\tfind_multipaths no\n\tretain_attached_hw_handler no\n\tdetect_prio no\n\thw_str_match no\n}\nblacklist {\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevice {\n\t\tvendor "DGC"\n\t\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390.*"\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "Universal Xport"\n\t}\n}\nblacklist_exceptions {\n}\ndevices {\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "NFINIDAT"\n\t\tproduct "InfiniBox.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_selector "round-robin 0"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\tprio "alua"\n\t\tfailback 30\n\t\trr_weight "priorities"\n\t\tno_path_retry "fail"\n\t\trr_min_io 1\n\t\trr_min_io_rq 1\n\t\tflush_on_last_del "yes"\n\t\tdev_loss_tmo 30\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "COMPELNT"\n\t\tproduct "Compellent Vol"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "APPLE*"\n\t\tproduct "Xserve RAID "\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "3PARdata"\n\t\tproduct "VV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "DEC"\n\t\tproduct "HSG80"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "A6189A"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "MSA VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2[02]12fc|MSA2012i"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2012sa|MSA23(12|24)(fc|i|sa)|MSA2000s VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "HSVX700"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "LOGICAL VOLUME.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "P2000 G3 FC|P2000G3 FC/iSCSI|P2000 G3 SAS|P2000 G3 iSCSI"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "DDN"\n\t\tproduct "SAN DataDirector"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "SYMMETRIX"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 6\n\t}\n\tdevice {\n\t\tvendor "DGC"\n\t\tproduct ".*"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "emc_clariion"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 emc"\n\t\tprio "emc"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "Invista"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 5\n\t}\n\tdevice {\n\t\tvendor "FSC"\n\t\tproduct "CentricStor"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "(HITACHI|HP)"\n\t\tproduct "OPEN-.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HITACHI"\n\t\tproduct "DF.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "hds"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "ProFibre 4000R"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD ECKD"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD FBA"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "1820N00"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "2810XIV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback 15\n\t\trr_weight "uniform"\n\t\trr_min_io 15\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "VDASD"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "3303      NVDISK"\n\t\tpath_grouping_policy "failover"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "NVDISK"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000i"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xx"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xxi"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NETAPP"\n\t\tproduct "LUN.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "3 queue_if_no_path pg_init_retries 50"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t\tflush_on_last_del "yes"\n\t\tdev_loss_tmo "infinity"\n\t\tuser_friendly_names no\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "NEXENTA"\n\t\tproduct "COMSTAR"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Nseries.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "Pillar"\n\t\tproduct "Axiom.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "IS.*"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NEC"\n\t\tproduct "DISK ARRAY"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "OPENstorage D280"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "(StorEdge 3510|T4)"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "STK6580_6780"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EUROLOGC"\n\t\tproduct "FC2502"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "PIVOT3"\n\t\tproduct "RAIGE VOLUME"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "CSM200_R"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "SUN_6180"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 1000\n\t\trr_min_io_rq 1\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "INF-01-00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "FLEXLINE 380"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "Intel"\n\t\tproduct "Multi-Flex"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "DataCore"\n\t\tproduct "SANmelody"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n}\nmultipaths {\n}'
MPATH_CONFIG_NO_NFINIDAT_CONFIG = 'defaults {\n\tverbosity 2\n\tpolling_interval 5\n\tmax_polling_interval 20\n\treassign_maps "yes"\n\tmultipath_dir "/lib64/multipath"\n\tpath_selector "service-time 0"\n\tpath_grouping_policy "failover"\n\tuid_attribute "ID_SERIAL"\n\tprio "const"\n\tprio_args ""\n\tfeatures "0"\n\tpath_checker "directio"\n\talias_prefix "mpath"\n\tfailback "manual"\n\trr_min_io 1000\n\trr_min_io_rq 1\n\tmax_fds 8192\n\trr_weight "uniform"\n\tqueue_without_daemon "no"\n\tpg_timeout "none"\n\tflush_on_last_del "no"\n\tuser_friendly_names "yes"\n\tfast_io_fail_tmo 5\n\tbindings_file "/etc/multipath/bindings"\n\twwids_file /etc/multipath/wwids\n\tlog_checker_err always\n\tfind_multipaths no\n\tretain_attached_hw_handler no\n\tdetect_prio no\n\thw_str_match no\n}\nblacklist {\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevice {\n\t\tvendor "DGC"\n\t\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390.*"\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "Universal Xport"\n\t}\n}\nblacklist_exceptions {\n}\ndevices {\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "COMPELNT"\n\t\tproduct "Compellent Vol"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "APPLE*"\n\t\tproduct "Xserve RAID "\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "3PARdata"\n\t\tproduct "VV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "DEC"\n\t\tproduct "HSG80"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "A6189A"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "MSA VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2[02]12fc|MSA2012i"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2012sa|MSA23(12|24)(fc|i|sa)|MSA2000s VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "HSVX700"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "LOGICAL VOLUME.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "P2000 G3 FC|P2000G3 FC/iSCSI|P2000 G3 SAS|P2000 G3 iSCSI"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "DDN"\n\t\tproduct "SAN DataDirector"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "SYMMETRIX"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 6\n\t}\n\tdevice {\n\t\tvendor "DGC"\n\t\tproduct ".*"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "emc_clariion"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 emc"\n\t\tprio "emc"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "Invista"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 5\n\t}\n\tdevice {\n\t\tvendor "FSC"\n\t\tproduct "CentricStor"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "(HITACHI|HP)"\n\t\tproduct "OPEN-.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HITACHI"\n\t\tproduct "DF.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "hds"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "ProFibre 4000R"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD ECKD"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD FBA"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "1820N00"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "2810XIV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback 15\n\t\trr_weight "uniform"\n\t\trr_min_io 15\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "VDASD"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "3303      NVDISK"\n\t\tpath_grouping_policy "failover"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "NVDISK"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000i"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xx"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xxi"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NETAPP"\n\t\tproduct "LUN.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "3 queue_if_no_path pg_init_retries 50"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t\tflush_on_last_del "yes"\n\t\tdev_loss_tmo "infinity"\n\t\tuser_friendly_names no\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "NEXENTA"\n\t\tproduct "COMSTAR"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Nseries.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "Pillar"\n\t\tproduct "Axiom.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "IS.*"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NEC"\n\t\tproduct "DISK ARRAY"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "OPENstorage D280"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "(StorEdge 3510|T4)"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "STK6580_6780"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EUROLOGC"\n\t\tproduct "FC2502"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "PIVOT3"\n\t\tproduct "RAIGE VOLUME"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "CSM200_R"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "SUN_6180"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 1000\n\t\trr_min_io_rq 1\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "INF-01-00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "FLEXLINE 380"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "Intel"\n\t\tproduct "Multi-Flex"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "DataCore"\n\t\tproduct "SANmelody"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n}\nmultipaths {\n}'
MPATH_CONFIG_BAD_GLOBAL_PARAMS = 'defaults {\n\tverbosity 2\n\tpolling_interval 5\n\tmax_polling_interval 20\n\treassign_maps "yes"\n\tmultipath_dir "/lib64/multipath"\n\tpath_selector "service-time 0"\n\tpath_grouping_policy "failover"\n\tuid_attribute "ID_SERIAL"\n\tprio "const"\n\tprio_args ""\n\tfeatures "0"\n\tpath_checker "directio"\n\talias_prefix "mpath"\n\tfailback "manual"\n\trr_min_io 1000\n\trr_min_io_rq 1\n\tmax_fds 666\n\trr_weight "uniform"\n\tqueue_without_daemon "no"\n\tpg_timeout "none"\n\tflush_on_last_del "no"\n\tuser_friendly_names "yes"\n\tfast_io_fail_tmo 5\n\tbindings_file "/etc/multipath/bindings"\n\twwids_file /etc/multipath/wwids\n\tlog_checker_err always\n\tfind_multipaths no\n\tretain_attached_hw_handler no\n\tdetect_prio no\n\thw_str_match no\n}\nblacklist {\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevice {\n\t\tvendor "DGC"\n\t\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390.*"\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "Universal Xport"\n\t}\n}\nblacklist_exceptions {\n}\ndevices {\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "NFINIDAT"\n\t\tproduct "InfiniBox.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_selector "round-robin 0"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\tprio "alua"\n\t\tfailback 30\n\t\trr_weight "priorities"\n\t\tno_path_retry "fail"\n\t\trr_min_io 1\n\t\trr_min_io_rq 1\n\t\tflush_on_last_del "yes"\n\t\tdev_loss_tmo 30\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "COMPELNT"\n\t\tproduct "Compellent Vol"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "APPLE*"\n\t\tproduct "Xserve RAID "\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "3PARdata"\n\t\tproduct "VV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "DEC"\n\t\tproduct "HSG80"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "A6189A"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "MSA VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2[02]12fc|MSA2012i"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2012sa|MSA23(12|24)(fc|i|sa)|MSA2000s VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "HSVX700"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "LOGICAL VOLUME.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "P2000 G3 FC|P2000G3 FC/iSCSI|P2000 G3 SAS|P2000 G3 iSCSI"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "DDN"\n\t\tproduct "SAN DataDirector"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "SYMMETRIX"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 6\n\t}\n\tdevice {\n\t\tvendor "DGC"\n\t\tproduct ".*"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "emc_clariion"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 emc"\n\t\tprio "emc"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "Invista"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 5\n\t}\n\tdevice {\n\t\tvendor "FSC"\n\t\tproduct "CentricStor"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "(HITACHI|HP)"\n\t\tproduct "OPEN-.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HITACHI"\n\t\tproduct "DF.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "hds"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "ProFibre 4000R"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD ECKD"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD FBA"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "1820N00"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "2810XIV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback 15\n\t\trr_weight "uniform"\n\t\trr_min_io 15\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "VDASD"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "3303      NVDISK"\n\t\tpath_grouping_policy "failover"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "NVDISK"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000i"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xx"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xxi"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NETAPP"\n\t\tproduct "LUN.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "3 queue_if_no_path pg_init_retries 50"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t\tflush_on_last_del "yes"\n\t\tdev_loss_tmo "infinity"\n\t\tuser_friendly_names no\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "NEXENTA"\n\t\tproduct "COMSTAR"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Nseries.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "Pillar"\n\t\tproduct "Axiom.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "IS.*"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NEC"\n\t\tproduct "DISK ARRAY"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "OPENstorage D280"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "(StorEdge 3510|T4)"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "STK6580_6780"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EUROLOGC"\n\t\tproduct "FC2502"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "PIVOT3"\n\t\tproduct "RAIGE VOLUME"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "CSM200_R"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "SUN_6180"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 1000\n\t\trr_min_io_rq 1\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "INF-01-00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "FLEXLINE 380"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "Intel"\n\t\tproduct "Multi-Flex"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "DataCore"\n\t\tproduct "SANmelody"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n}\nmultipaths {\n}'
MPATH_CONFIG_NFINIDAT_IN_BLACKLIST = 'defaults {\n\tverbosity 2\n\tpolling_interval 5\n\tmax_polling_interval 20\n\treassign_maps "yes"\n\tmultipath_dir "/lib64/multipath"\n\tpath_selector "service-time 0"\n\tpath_grouping_policy "failover"\n\tuid_attribute "ID_SERIAL"\n\tprio "const"\n\tprio_args ""\n\tfeatures "0"\n\tpath_checker "directio"\n\talias_prefix "mpath"\n\tfailback "manual"\n\trr_min_io 1000\n\trr_min_io_rq 1\n\tmax_fds 8192\n\trr_weight "uniform"\n\tqueue_without_daemon "no"\n\tpg_timeout "none"\n\tflush_on_last_del "no"\n\tuser_friendly_names "yes"\n\tfast_io_fail_tmo 5\n\tbindings_file "/etc/multipath/bindings"\n\twwids_file /etc/multipath/wwids\n\tlog_checker_err always\n\tfind_multipaths no\n\tretain_attached_hw_handler no\n\tdetect_prio no\n\thw_str_match no\n}\nblacklist {\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"\n\tdevnode "^(td|hd)[a-z]"\n\tdevnode "^dcssblk[0-9]*"\n\tdevice {\n\t\tvendor "DGC"\n\t\t}\n\tdevice {\n\t\tvendor "NFINIDAT"\n\t\tproduct "InfiniBox.*"\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390.*"\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "Universal Xport"\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "Universal Xport"\n\t}\n}\nblacklist_exceptions {\n}\ndevices {\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "NFINIDAT"\n\t\tproduct "InfiniBox.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_selector "round-robin 0"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\tprio "alua"\n\t\tfailback 30\n\t\trr_weight "priorities"\n\t\tno_path_retry "fail"\n\t\trr_min_io 1\n\t\trr_min_io_rq 1\n\t\tflush_on_last_del "yes"\n\t\tdev_loss_tmo 30\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "COMPELNT"\n\t\tproduct "Compellent Vol"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "APPLE*"\n\t\tproduct "Xserve RAID "\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "3PARdata"\n\t\tproduct "VV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "DEC"\n\t\tproduct "HSG80"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "A6189A"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "(MSA|HSV)1.0.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "hp_sw"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 hp_sw"\n\t\tprio "hp_sw"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "MSA VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "(COMPAQ|HP)"\n\t\tproduct "HSV1[01]1|HSV2[01]0|HSV3[046]0|HSV4[05]0"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2[02]12fc|MSA2012i"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "MSA2012sa|MSA23(12|24)(fc|i|sa)|MSA2000s VOLUME"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "HSVX700"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "LOGICAL VOLUME.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 12\n\t}\n\tdevice {\n\t\tvendor "HP"\n\t\tproduct "P2000 G3 FC|P2000G3 FC/iSCSI|P2000 G3 SAS|P2000 G3 iSCSI"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 18\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "DDN"\n\t\tproduct "SAN DataDirector"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "SYMMETRIX"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 6\n\t}\n\tdevice {\n\t\tvendor "DGC"\n\t\tproduct ".*"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "emc_clariion"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 emc"\n\t\tprio "emc"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "EMC"\n\t\tproduct "Invista"\n\t\tproduct_blacklist "LUNZ"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\tno_path_retry 5\n\t}\n\tdevice {\n\t\tvendor "FSC"\n\t\tproduct "CentricStor"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "FUJITSU"\n\t\tproduct "ETERNUS_DX(L|400|8000)"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 10\n\t}\n\tdevice {\n\t\tvendor "(HITACHI|HP)"\n\t\tproduct "OPEN-.*"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "HITACHI"\n\t\tproduct "DF.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "hds"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "ProFibre 4000R"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1722-600"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1724"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1726"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 300\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1742"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1745|^1746"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "2 pg_init_retries 50"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1814"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1815"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1818"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3526"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^3542"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105800"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2105F20"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^1750500"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2107900"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^2145"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD ECKD"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "S/390 DASD FBA"\n\t\tproduct_blacklist "S/390.*"\n\t\tpath_grouping_policy "multibus"\n\t\tuid_attribute "ID_UID"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "^IPR.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "1820N00"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "2810XIV"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback 15\n\t\trr_weight "uniform"\n\t\trr_min_io 15\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "VDASD"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "3303      NVDISK"\n\t\tpath_grouping_policy "failover"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "AIX"\n\t\tproduct "NVDISK"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 60\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD3000i"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xx"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "DELL"\n\t\tproduct "MD32xxi"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NETAPP"\n\t\tproduct "LUN.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "3 queue_if_no_path pg_init_retries 50"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t\tflush_on_last_del "yes"\n\t\tdev_loss_tmo "infinity"\n\t\tuser_friendly_names no\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "NEXENTA"\n\t\tproduct "COMSTAR"\n\t\tpath_grouping_policy "group_by_serial"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "IBM"\n\t\tproduct "Nseries.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "ontap"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\trr_min_io 128\n\t}\n\tdevice {\n\t\tvendor "Pillar"\n\t\tproduct "Axiom.*"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[13]00"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "TP9[45]00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SGI"\n\t\tproduct "IS.*"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 15\n\t}\n\tdevice {\n\t\tvendor "NEC"\n\t\tproduct "DISK ARRAY"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "OPENstorage D280"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "(StorEdge 3510|T4)"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "STK6580_6780"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "EUROLOGC"\n\t\tproduct "FC2502"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "directio"\n\t\tfeatures "0"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t}\n\tdevice {\n\t\tvendor "PIVOT3"\n\t\tproduct "RAIGE VOLUME"\n\t\tpath_grouping_policy "multibus"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "const"\n\t\trr_weight "uniform"\n\t\trr_min_io 100\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "CSM200_R"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "LCSM100_[IEFS]"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "0"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "SUN"\n\t\tproduct "SUN_6180"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t\trr_min_io 1000\n\t\trr_min_io_rq 1\n\t}\n\tdevice {\n\t\tvendor "(NETAPP|LSI|ENGENIO)"\n\t\tproduct "INF-01-00"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "3 pg_init_retries 50 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry 30\n\t\tretain_attached_hw_handler yes\n\t\tdetect_prio yes\n\t}\n\tdevice {\n\t\tvendor "STK"\n\t\tproduct "FLEXLINE 380"\n\t\tproduct_blacklist "Universal Xport"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "rdac"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 rdac"\n\t\tprio "rdac"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "Intel"\n\t\tproduct "Multi-Flex"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "1 alua"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n\tdevice {\n\t\tvendor "DataCore"\n\t\tproduct "SANmelody"\n\t\tpath_grouping_policy "group_by_prio"\n\t\tpath_checker "tur"\n\t\tfeatures "1 queue_if_no_path"\n\t\thardware_handler "0"\n\t\tprio "alua"\n\t\tfailback immediate\n\t\trr_weight "uniform"\n\t\tno_path_retry "queue"\n\t}\n}\nmultipaths {\n}'

@contextmanager
def initial_multipath_config_context(initial_multipath_config):
    def _send_and_receive_side_effect(*args, **kwargs):
        if args[0] == 'show config':
            return initial_multipath_config
        return 'ok'
    with patch('infi.multipathtools.client.MultipathClient._send_and_receive', MagicMock(side_effect=_send_and_receive_side_effect)):
        with patch('infi.vendata.powertools.settings.cases.linux.base_cases.boot_from_san.ask_for_reboot'):
            func_name = 'infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.MultipathConfigBase._get_infinidat_wwids'
            with patch(func_name) as get_infinidat_wwids:
                get_infinidat_wwids.return_value = INFINIDAT_WWIDS
                yield


class TestDeviceMapperSettings(FakeFSTestCase):
    @classmethod
    def setUpClass(cls):
        cls._save_source_files_for_pdb()

    def setUp(self):
        from platform import system
        if 'windows' in system().lower():
            self.skipTest("Skipping on windows since it's a linux module")

        self.setUpPyfakefs()
        # this handles relative imports
        self._original_shutil = sys.modules['shutil']
        self._original_os = sys.modules['os']
        self._original_glob = sys.modules['glob']
        sys.modules['shutil'] = self._stubber.fake_shutil
        sys.modules['os'] = self._stubber.fake_os
        sys.modules['glob'] = self._stubber.fake_glob

        self.fs.CreateFile(GLOBAL_MULTIPATH_CONF_PATH, contents='')
        self.fs.CreateFile(device_mapper.MULTIPATH_CONF_PATH, contents=RECOMMENDED_MULTIPATH_CONF_CONTENT)
        self._put_source_files_in_fakefs()

    @classmethod
    def _save_source_files_for_pdb(cls):
        cls._module_data = {}
        modules = dict(sys.modules)
        for module_name, module in modules.iteritems():
            if module and hasattr(module, '__file__'):
                filename = module.__file__.replace('.pyc', '.py')
                filename = filename.replace('.pyo', '.py')
                try:
                    data = open(filename, 'r').read()
                    cls._module_data[module.__file__] = data
                    cls._module_data[filename] = data
                except:
                    pass

    def _put_source_files_in_fakefs(self):
        for module, data in self._module_data.iteritems():
            self.fs.CreateFile(module, contents=data)

    def _assert_file_nonexistent_or_empty(self, filename):
        self.assertTrue(not FakeOsModule(self.fs).path.exists(filename) or
                        FakeFileOpen(self.fs)(filename).read() == '')

    def _assert_file_exists_and_nonempty(self, filename):
        self.assertTrue(FakeOsModule(self.fs).path.exists(filename) and
                        FakeFileOpen(self.fs)(filename).read() != '')

    def _assert_new_config(self, content):
        from infi.multipathtools.config import Configuration as MultipathConfiguration
        from . import workaround_missing_vid_pid
        new_config = workaround_missing_vid_pid(MultipathConfiguration.from_multipathd_conf(FakeFileOpen(self.fs)(GLOBAL_MULTIPATH_CONF_PATH).read()))
        expected_config = workaround_missing_vid_pid(MultipathConfiguration.from_multipathd_conf(content))
        self.assertTrue(set((x.product.strip('"'), x.vendor.strip('"')) for x in new_config.blacklist.device) == set((x.product.strip('"'), x.vendor.strip('"')) for x in expected_config.blacklist.device))
        self.assertTrue(set((x.product.strip('"'), x.vendor.strip('"')) for x in new_config.whitelist.device) == set((x.product.strip('"'), x.vendor.strip('"')) for x in expected_config.whitelist.device))
        self.assertTrue(set((a[0], a[1].strip('"')) for a in new_config.attributes.items()) == set((a[0], a[1].strip('"')) for a in expected_config.attributes.items()))

    @contextmanager
    def pretend_veritas_is_installed_context(self):
        self.fs.CreateFile('/usr/sbin/vxdmpadm', contents='')
        yield

    def tearDown(self):
        sys.modules['glob'] = self._original_glob
        sys.modules['os'] = self._original_os
        sys.modules['shutil'] = self._original_shutil
        self.tearDownPyfakefs()

    def test_multipath_config_defaults(self):
        with patch('infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.MultipathConfigDefaults._ensure_fixed', MagicMock(return_value=True)):
            with initial_multipath_config_context(MPATH_CONFIG_BAD_GLOBAL_PARAMS):
                test_case = device_mapper.MultipathConfigDefaults(None)
                with self.assertRaises(case.Failure):
                    test_case.run_test()
                test_case.run_fix()
                self._assert_file_exists_and_nonempty(GLOBAL_MULTIPATH_CONF_PATH)
                self._assert_new_config(MPATH_CONFIG_FIXED)

    def test_multipath_config_ok(self):
        with initial_multipath_config_context(MPATH_CONFIG_FIXED):
            test_case = device_mapper.MultipathConfigDefaults(None)
            test_case.run_test()

    def test_multipath_config_recommended(self):
        with patch('infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.MultipathConfigRecommened._ensure_fixed', MagicMock(return_value=True)):
            with initial_multipath_config_context(MPATH_CONFIG_NO_NFINIDAT_CONFIG):
                test_case = device_mapper.MultipathConfigRecommened(None)
                with self.assertRaises(case.Failure):
                    test_case.run_test()
                test_case.run_fix()
                self._assert_new_config(MPATH_CONFIG_FIXED)

    def test_multipath_config_recommended_veritas_rules_exist(self):
        with patch('infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.MultipathConfigRecommened._ensure_fixed', MagicMock(return_value=True)):
            with self.pretend_veritas_is_installed_context():
                with initial_multipath_config_context(MPATH_CONFIG_FIXED):
                    test_case = device_mapper.MultipathConfigRecommened(None)
                    with self.assertRaises(case.Failure):
                        test_case.run_test()
                    test_case.run_fix()
                    self._assert_new_config(MPATH_CONFIG_NO_NFINIDAT_CONFIG)

    def test_multipath_config_recommended_veritas(self):
        with self.pretend_veritas_is_installed_context():
            with initial_multipath_config_context(MPATH_CONFIG_NO_NFINIDAT_CONFIG):
                test_case = device_mapper.MultipathConfigRecommened(None)
                test_case.run_test()

    def test_multipath_infinidat_exclusions_veritas(self):
        with self.pretend_veritas_is_installed_context():
            with initial_multipath_config_context(MPATH_CONFIG_NFINIDAT_IN_BLACKLIST):
                test_case = device_mapper.MultipathConfigInfinidatExclusions(None)
                test_case.run_test()

    def test_multipath_infinidat_exclusions_veritas_no_exclusion(self):
        with patch('infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.MultipathConfigInfinidatExclusions._ensure_fixed', MagicMock(return_value=True)):
            with self.pretend_veritas_is_installed_context():
                with initial_multipath_config_context(MPATH_CONFIG_FIXED):
                    test_case = device_mapper.MultipathConfigInfinidatExclusions(None)
                    with self.assertRaises(case.Failure):
                        test_case.run_test()
                    test_case.run_fix()
                    self._assert_new_config(MPATH_CONFIG_NFINIDAT_IN_BLACKLIST)

    def test_multipath_infinidat_exclusions_ok(self):
        with initial_multipath_config_context(MPATH_CONFIG_FIXED):
            test_case = device_mapper.MultipathConfigInfinidatExclusions(None)
            test_case.run_test()

    def test_multipath_infinidat_exclusions_exist(self):
        with patch('infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.MultipathConfigInfinidatExclusions._ensure_fixed', MagicMock(return_value=True)):
            with initial_multipath_config_context(MPATH_CONFIG_NFINIDAT_IN_BLACKLIST):
                test_case = device_mapper.MultipathConfigInfinidatExclusions(None)
                with self.assertRaises(case.Failure):
                    test_case.run_test()
                test_case.run_fix()
                self._assert_new_config(MPATH_CONFIG_FIXED)


def _get_ignore_fake_config():
    return {'readiness.cases.coexistence': {'ignore_powerpath': True},
            'readiness.cases.coexistence.ignore_powerpath': True}

def _get_no_ignore_fake_config():
    return {'readiness.cases.coexistence': {'ignore_powerpath': False},
            'readiness.cases.coexistence.ignore_powerpath': False}

class PowerPathDetectionTestCase(FakeFSTestCase):
    def setUp(self):
        from platform import system
        if 'linux' not in system().lower():
            self.skipTest("PowerPath is a linux module")
        from infi.pkgmgr import get_package_manager
        from infi.vendata.powertools import conf
        self._package_manager = get_package_manager()
        self._conf = conf
        self.setUpPyfakefs()

    def tearDown(self):
        self.tearDownPyfakefs()

    def test_powerpath_detection(self):
        import infi.pkgmgr
        with patch.object(infi.pkgmgr, 'get_package_manager', new=lambda: self._package_manager):
            with patch.object(self._package_manager, 'is_package_installed', new=lambda module: False):
                self.assertEquals(powerpath.PowerPath().is_powerpath_installed(), False)
                self.fs.CreateFile("/sbin/powermt")
                self.assertEquals(powerpath.PowerPath().is_powerpath_installed(), True)

    def test_powerpath_ignore_flag(self):
        import infi.pkgmgr
        with patch.object(infi.pkgmgr, 'get_package_manager', new=lambda: self._package_manager):
            with patch.object(self._package_manager, 'is_package_installed', new=lambda module: False):
                with patch.object(self._conf, 'get_configuration', new=_get_ignore_fake_config):
                    self.assertEquals(powerpath.PowerPath().should_warn(), False)
                    self.assertEquals(powerpath.PowerPath().should_fail(), False)
                    self.fs.CreateFile("/sbin/powermt")
                    self.assertEquals(powerpath.PowerPath().should_warn(), True)
                    self.assertEquals(powerpath.PowerPath().should_fail(), False)
                with patch.object(self._conf, 'get_configuration', new=_get_no_ignore_fake_config):
                    self.assertEquals(powerpath.PowerPath().should_warn(), False)
                    self.assertEquals(powerpath.PowerPath().should_fail(), True)
